@@ -24,22 +24,26 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include <gstfakesrc.h>
+#ifdef HAVE_CONFIG_H
+#  include "config.h"
+#endif
+
+#include "gstfakesrc.h"
 
 #define DEFAULT_SIZEMIN		0
 #define DEFAULT_SIZEMAX		4096
 #define DEFAULT_PARENTSIZE	4096*10
 
-GstElementDetails gst_fakesrc_details = {
+GST_DEBUG_CATEGORY_STATIC (gst_fakesrc_debug);
+#define GST_CAT_DEFAULT gst_fakesrc_debug
+
+GstElementDetails gst_fakesrc_details = GST_ELEMENT_DETAILS (
   "Fake Source",
   "Source",
-  "LGPL",
   "Push empty (no data) buffers around",
-  VERSION,
-  "Erik Walthinsen <omega@cse.ogi.edu>\n"
-  "Wim Taymans <wim.taymans@chello.be>",
-  "(C) 1999",
-};
+  "Erik Walthinsen <omega@cse.ogi.edu>, "
+  "Wim Taymans <wim.taymans@chello.be>"
+);
 
 
 /* FakeSrc signals and args */
@@ -62,6 +66,7 @@ enum {
   ARG_PATTERN,
   ARG_NUM_BUFFERS,
   ARG_EOS,
+  ARG_SIGNAL_HANDOFFS,
   ARG_SILENT,
   ARG_DUMP,
   ARG_PARENTSIZE,
@@ -72,7 +77,7 @@ GST_PAD_TEMPLATE_FACTORY (fakesrc_src_factory,
   "src%d",
   GST_PAD_SRC,
   GST_PAD_REQUEST,
-  NULL                  /* no caps */
+  GST_CAPS_ANY
 );
 
 #define GST_TYPE_FAKESRC_OUTPUT (gst_fakesrc_output_get_type())
@@ -93,6 +98,8 @@ gst_fakesrc_output_get_type (void)
   };
   if (!fakesrc_output_type) {
     fakesrc_output_type = g_enum_register_static ("GstFakeSrcOutput", fakesrc_output);
+  
+    GST_DEBUG_CATEGORY_INIT (gst_fakesrc_debug, "fakesrc", 0, "fakesrc element");
   }
   return fakesrc_output_type;
 }
@@ -150,10 +157,11 @@ gst_fakesrc_filltype_get_type (void)
   return fakesrc_filltype_type;
 }
 
+static void		gst_fakesrc_base_init		(gpointer g_class);
 static void		gst_fakesrc_class_init		(GstFakeSrcClass *klass);
 static void		gst_fakesrc_init		(GstFakeSrc *fakesrc);
 
-static GstPad* 		gst_fakesrc_request_new_pad 	(GstElement *element, GstPadTemplate *templ);
+static GstPad* 		gst_fakesrc_request_new_pad 	(GstElement *element, GstPadTemplate *templ,const gchar *unused);
 static void 		gst_fakesrc_update_functions 	(GstFakeSrc *src);
 static void		gst_fakesrc_set_property	(GObject *object, guint prop_id, 
 							 const GValue *value, GParamSpec *pspec);
@@ -162,7 +170,7 @@ static void		gst_fakesrc_get_property	(GObject *object, guint prop_id,
 
 static GstElementStateReturn gst_fakesrc_change_state 	(GstElement *element);
 
-static GstBuffer*	gst_fakesrc_get			(GstPad *pad);
+static GstData*	gst_fakesrc_get			(GstPad *pad);
 static void 		gst_fakesrc_loop		(GstElement *element);
 
 static GstElementClass *parent_class = NULL;
@@ -176,7 +184,7 @@ gst_fakesrc_get_type (void)
   if (!fakesrc_type) {
     static const GTypeInfo fakesrc_info = {
       sizeof(GstFakeSrcClass),
-      NULL,
+      gst_fakesrc_base_init,
       NULL,
       (GClassInitFunc)gst_fakesrc_class_init,
       NULL,
@@ -190,6 +198,14 @@ gst_fakesrc_get_type (void)
   return fakesrc_type;
 }
 
+static void
+gst_fakesrc_base_init (gpointer g_class)
+{
+  GstElementClass *gstelement_class = GST_ELEMENT_CLASS (g_class); 
+  
+  gst_element_class_set_details (gstelement_class, &gst_fakesrc_details);
+  gst_element_class_add_pad_template (gstelement_class, GST_PAD_TEMPLATE_GET (fakesrc_src_factory));
+}
 static void
 gst_fakesrc_class_init (GstFakeSrcClass *klass) 
 {
@@ -243,6 +259,9 @@ gst_fakesrc_class_init (GstFakeSrcClass *klass)
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_SILENT,
     g_param_spec_boolean ("silent", "Silent", "Don't produce last_message events",
                           FALSE, G_PARAM_READWRITE));
+  g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_SIGNAL_HANDOFFS,
+    g_param_spec_boolean ("signal-handoffs", "Signal handoffs", "Send a signal before pushing the buffer",
+                          FALSE, G_PARAM_READWRITE));
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_DUMP,
     g_param_spec_boolean ("dump", "Dump", "Dump produced bytes to stdout",
                           FALSE, G_PARAM_READWRITE));
@@ -250,8 +269,8 @@ gst_fakesrc_class_init (GstFakeSrcClass *klass)
   gst_fakesrc_signals[SIGNAL_HANDOFF] =
     g_signal_new ("handoff", G_TYPE_FROM_CLASS (klass), G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (GstFakeSrcClass, handoff), NULL, NULL,
-                  g_cclosure_marshal_VOID__POINTER, G_TYPE_NONE, 1,
-                  G_TYPE_POINTER);
+                  gst_marshal_VOID__POINTER_OBJECT, G_TYPE_NONE, 2,
+                  GST_TYPE_BUFFER, GST_TYPE_PAD);
 
   gobject_class->set_property = GST_DEBUG_FUNCPTR (gst_fakesrc_set_property);
   gobject_class->get_property = GST_DEBUG_FUNCPTR (gst_fakesrc_get_property);
@@ -279,6 +298,7 @@ gst_fakesrc_init (GstFakeSrc *fakesrc)
   fakesrc->rt_num_buffers = -1;
   fakesrc->buffer_count = 0;
   fakesrc->silent = FALSE;
+  fakesrc->signal_handoffs = FALSE;
   fakesrc->dump = FALSE;
   fakesrc->pattern_byte = 0x00;
   fakesrc->need_flush = FALSE;
@@ -293,7 +313,7 @@ gst_fakesrc_init (GstFakeSrc *fakesrc)
 }
 
 static GstPad*
-gst_fakesrc_request_new_pad (GstElement *element, GstPadTemplate *templ)
+gst_fakesrc_request_new_pad (GstElement *element, GstPadTemplate *templ, const gchar *unused)
 {
   gchar *name;
   GstPad *srcpad;
@@ -475,7 +495,7 @@ gst_fakesrc_set_property (GObject *object, guint prop_id, const GValue *value, G
           src->pool = gst_buffer_pool_get_default (src->sizemax, 10);
       } else {
         if (src->pool) {
-          gst_buffer_pool_free (src->pool);
+          gst_buffer_pool_unref (src->pool);
           src->pool = NULL;
         }
       }
@@ -502,11 +522,14 @@ gst_fakesrc_set_property (GObject *object, guint prop_id, const GValue *value, G
       break;
     case ARG_EOS:
       src->eos = g_value_get_boolean (value);
-      GST_INFO (0, "will EOS on next buffer");
+      GST_INFO ( "will EOS on next buffer");
       break;
     case ARG_SILENT:
       src->silent = g_value_get_boolean (value);
       break;
+    case ARG_SIGNAL_HANDOFFS:
+      src->signal_handoffs = g_value_get_boolean (value);
+      break;
     case ARG_DUMP:
       src->dump = g_value_get_boolean (value);
       break;
@@ -565,6 +588,9 @@ gst_fakesrc_get_property (GObject *object, guint prop_id, GValue *value, GParamS
     case ARG_SILENT:
       g_value_set_boolean (value, src->silent);
       break;
+    case ARG_SIGNAL_HANDOFFS:
+      g_value_set_boolean (value, src->signal_handoffs);
+      break;
     case ARG_DUMP:
       g_value_set_boolean (value, src->dump);
       break;
@@ -716,7 +742,7 @@ gst_fakesrc_create_buffer (GstFakeSrc *src)
   return buf;
 }
 
-static GstBuffer *
+static GstData *
 gst_fakesrc_get(GstPad *pad)
 {
   GstFakeSrc *src;
@@ -724,28 +750,28 @@ gst_fakesrc_get(GstPad *pad)
 
   g_return_val_if_fail (pad != NULL, NULL);
 
-  src = GST_FAKESRC (gst_pad_get_parent (pad));
+  src = GST_FAKESRC (GST_OBJECT_PARENT (pad));
 
   g_return_val_if_fail (GST_IS_FAKESRC (src), NULL);
 
   if (src->need_flush) {
     src->need_flush = FALSE;
-    return GST_BUFFER(gst_event_new (GST_EVENT_FLUSH));
+    return GST_DATA(gst_event_new (GST_EVENT_FLUSH));
   }
 
   if (src->buffer_count == src->segment_end) {
     if (src->segment_loop) {
-      return GST_BUFFER(gst_event_new (GST_EVENT_SEGMENT_DONE));
+      return GST_DATA(gst_event_new (GST_EVENT_SEGMENT_DONE));
     }
     else {
       gst_element_set_eos (GST_ELEMENT (src));
-      return GST_BUFFER(gst_event_new (GST_EVENT_EOS));
+      return GST_DATA(gst_event_new (GST_EVENT_EOS));
     }
   }
 
   if (src->rt_num_buffers == 0) {
     gst_element_set_eos (GST_ELEMENT (src));
-    return GST_BUFFER(gst_event_new (GST_EVENT_EOS));
+    return GST_DATA(gst_event_new (GST_EVENT_EOS));
   }
   else {
     if (src->rt_num_buffers > 0)
@@ -753,8 +779,8 @@ gst_fakesrc_get(GstPad *pad)
   }
 
   if (src->eos) {
-    GST_INFO (0, "fakesrc is setting eos on pad");
-    return GST_BUFFER(gst_event_new (GST_EVENT_EOS));
+    GST_INFO ( "fakesrc is setting eos on pad");
+    return GST_DATA(gst_event_new (GST_EVENT_EOS));
   }
 
   buf = gst_fakesrc_create_buffer (src);
@@ -770,12 +796,14 @@ gst_fakesrc_get(GstPad *pad)
     g_object_notify (G_OBJECT (src), "last_message");
   }
 
-  GST_DEBUG_ELEMENT (GST_CAT_DATAFLOW, src, "pre handoff emit");
-  g_signal_emit (G_OBJECT (src), gst_fakesrc_signals[SIGNAL_HANDOFF], 0,
+  if (src->signal_handoffs) {
+    GST_LOG_OBJECT (src, "pre handoff emit");
+    g_signal_emit (G_OBJECT (src), gst_fakesrc_signals[SIGNAL_HANDOFF], 0,
                    buf, pad);
-  GST_DEBUG_ELEMENT (GST_CAT_DATAFLOW, src, "post handoff emit");
+    GST_LOG_OBJECT (src, "post handoff emit");
+  }
 
-  return buf;
+  return GST_DATA (buf);
 }
 
 /**
@@ -799,10 +827,10 @@ gst_fakesrc_loop(GstElement *element)
 
   while (pads) {
     GstPad *pad = GST_PAD (pads->data);
-    GstBuffer *buf;
+    GstData *data;
 
-    buf = gst_fakesrc_get (pad);
-    gst_pad_push (pad, buf);
+    data = gst_fakesrc_get (pad);
+    gst_pad_push (pad, data);
 
     if (src->eos) {
       return;
@@ -859,11 +887,3 @@ gst_fakesrc_change_state (GstElement *element)
   return GST_STATE_SUCCESS;
 }
 
-gboolean
-gst_fakesrc_factory_init (GstElementFactory *factory)
-{
-  gst_element_factory_add_pad_template (factory, GST_PAD_TEMPLATE_GET (fakesrc_src_factory));
-
-  return TRUE;
-}
-
