@@ -20,8 +20,11 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include <gst/gst.h>
+#ifdef HAVE_CONFIG_H
+#  include "config.h"
+#endif
 
+#include <gst/gst.h>
 #include "gstfilesrc.h"
 
 #include <stdio.h>
@@ -69,31 +72,19 @@
  */
 
 
-GstElementDetails gst_filesrc_details = {
+GST_DEBUG_CATEGORY_STATIC (gst_filesrc_debug);
+#define GST_CAT_DEFAULT gst_filesrc_debug
+
+GstElementDetails gst_filesrc_details = GST_ELEMENT_DETAILS (
   "File Source",
   "Source/File",
-  "LGPL",
   "Read from arbitrary point in a file",
-  VERSION,
-  "Erik Walthinsen <omega@cse.ogi.edu>",
-  "(C) 1999",
-};
+  "Erik Walthinsen <omega@cse.ogi.edu>"
+);
 
 #define DEFAULT_BLOCKSIZE 	4*1024
 #define DEFAULT_MMAPSIZE 	4*1024*1024
 
-#ifdef G_HAVE_ISO_VARARGS
-
-/* #define fs_print(...) g_print(__VA_ARGS__)  */
-#define fs_print(...)
-
-#elif defined(G_HAVE_GNUC_VARARGS)
-
-/* #define fs_print(format,args...) g_print(format, ## args)  */
-#define fs_print(format,args...)
-
-#endif
-
 /* FileSrc signals and args */
 enum {
   /* FILL ME */
@@ -127,6 +118,7 @@ GST_PAD_FORMATS_FUNCTION (gst_filesrc_get_formats,
   GST_FORMAT_BYTES
 )
 
+static void		gst_filesrc_base_init		(gpointer g_class);
 static void		gst_filesrc_class_init		(GstFileSrcClass *klass);
 static void		gst_filesrc_init		(GstFileSrc *filesrc);
 static void 		gst_filesrc_dispose 		(GObject *object);
@@ -136,13 +128,16 @@ static void		gst_filesrc_set_property	(GObject *object, guint prop_id,
 static void		gst_filesrc_get_property	(GObject *object, guint prop_id, 
 							 GValue *value, GParamSpec *pspec);
 
-static GstBuffer *	gst_filesrc_get			(GstPad *pad);
+static gboolean		gst_filesrc_check_filesize	(GstFileSrc *src); 
+static GstData *	gst_filesrc_get			(GstPad *pad);
 static gboolean 	gst_filesrc_srcpad_event 	(GstPad *pad, GstEvent *event);
 static gboolean 	gst_filesrc_srcpad_query 	(GstPad *pad, GstQueryType type,
 		         				 GstFormat *format, gint64 *value);
 
 static GstElementStateReturn	gst_filesrc_change_state	(GstElement *element);
 
+static void		gst_filesrc_uri_handler_init	(gpointer g_iface, gpointer iface_data);
+
 
 static GstElementClass *parent_class = NULL;
 /*static guint gst_filesrc_signals[LAST_SIGNAL] = { 0 };*/
@@ -154,7 +149,8 @@ gst_filesrc_get_type(void)
 
   if (!filesrc_type) {
     static const GTypeInfo filesrc_info = {
-      sizeof(GstFileSrcClass),      NULL,
+      sizeof(GstFileSrcClass), 
+      gst_filesrc_base_init,
       NULL,
       (GClassInitFunc)gst_filesrc_class_init,
       NULL,
@@ -163,21 +159,36 @@ gst_filesrc_get_type(void)
       0,
       (GInstanceInitFunc)gst_filesrc_init,
     };
+    static const GInterfaceInfo urihandler_info = {
+      gst_filesrc_uri_handler_init,
+      NULL,
+      NULL
+    };
     filesrc_type = g_type_register_static (GST_TYPE_ELEMENT, "GstFileSrc", &filesrc_info, 0);
+    
+    g_type_add_interface_static (filesrc_type, GST_TYPE_URI_HANDLER, &urihandler_info);
+    
+    GST_DEBUG_CATEGORY_INIT (gst_filesrc_debug, "filesrc", 0, "filesrc element");
   }
   return filesrc_type;
 }
 
+static void
+gst_filesrc_base_init (gpointer g_class)
+{
+  GstElementClass *gstelement_class = GST_ELEMENT_CLASS (g_class);
+
+  gst_element_class_set_details (gstelement_class, &gst_filesrc_details);
+}
 static void
 gst_filesrc_class_init (GstFileSrcClass *klass)
 {
-  GObjectClass *gobject_class;
-  GstElementClass *gstelement_class;
+  GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
+  GstElementClass *gstelement_class = GST_ELEMENT_CLASS (klass);
 
   gobject_class = (GObjectClass*)klass;
-  gstelement_class = (GstElementClass*)klass;
 
-  parent_class = g_type_class_ref (GST_TYPE_ELEMENT);
+  parent_class = g_type_class_peek_parent (klass);
 
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_FD,
     g_param_spec_int ("fd", "File-descriptor", "File-descriptor for the file being mmap()d",
@@ -261,8 +272,32 @@ gst_filesrc_dispose (GObject *object)
   g_mutex_free (src->map_regions_lock);
   if (src->filename)
     g_free (src->filename);
+  if (src->uri)
+    g_free (src->uri);
 }
 
+static gboolean
+gst_filesrc_set_location (GstFileSrc *src, const gchar *location)
+{
+  /* the element must be stopped in order to do this */
+  if (GST_STATE (src) == GST_STATE_PLAYING)
+    return FALSE;
+
+  if (src->filename) g_free (src->filename);
+  if (src->uri) g_free (src->uri);
+  /* clear the filename if we get a NULL (is that possible?) */
+  if (location == NULL) {
+    src->filename = NULL;
+    src->uri = NULL;
+  } else {
+    src->filename = g_strdup (location);
+    src->uri = gst_uri_construct ("file", src->filename);
+  }
+  g_object_notify (G_OBJECT (src), "location");
+  gst_uri_handler_new_uri (GST_URI_HANDLER (src), src->uri);
+
+  return TRUE;
+}
 
 static void
 gst_filesrc_set_property (GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec)
@@ -276,19 +311,7 @@ gst_filesrc_set_property (GObject *object, guint prop_id, const GValue *value, G
 
   switch (prop_id) {
     case ARG_LOCATION:
-      /* the element must be stopped in order to do this */
-      g_return_if_fail (GST_STATE (src) < GST_STATE_PLAYING);
-
-      if (src->filename) g_free (src->filename);
-      /* clear the filename if we get a NULL (is that possible?) */
-      if (g_value_get_string (value) == NULL) {
-        gst_element_set_state (GST_ELEMENT (object), GST_STATE_NULL);
-        src->filename = NULL;
-      /* otherwise set the new filename */
-      } else {
-        src->filename = g_strdup (g_value_get_string (value));
-      }
-      g_object_notify (G_OBJECT (src), "location");
+      gst_filesrc_set_location (src, g_value_get_string (value));
       break;
     case ARG_BLOCKSIZE:
       src->block_size = g_value_get_ulong (value);
@@ -299,7 +322,7 @@ gst_filesrc_set_property (GObject *object, guint prop_id, const GValue *value, G
         src->mapsize = g_value_get_ulong (value);
         g_object_notify (G_OBJECT (src), "mmapsize");
       } else {
-        GST_INFO (0, "invalid mapsize, must a multiple of pagesize, which is %d", 
+        GST_INFO_OBJECT (src, "invalid mapsize, must a multiple of pagesize, which is %d", 
 	          src->pagesize);
       }
       break;
@@ -349,8 +372,8 @@ gst_filesrc_free_parent_mmap (GstBuffer *buf)
 {
   GstFileSrc *src = GST_FILESRC (GST_BUFFER_POOL_PRIVATE (buf));
 
-  fs_print ("freeing mmap()d buffer at %d+%d\n", 
-	    GST_BUFFER_OFFSET (buf), GST_BUFFER_SIZE (buf));
+  GST_LOG_OBJECT (src, "freeing mmap()d buffer at %"G_GUINT64_FORMAT"+%u", 
+		  GST_BUFFER_OFFSET (buf), GST_BUFFER_SIZE (buf));
 
   /* remove the buffer from the list of available mmap'd regions */
   g_mutex_lock (src->map_regions_lock);
@@ -369,13 +392,14 @@ gst_filesrc_free_parent_mmap (GstBuffer *buf)
   munmap (GST_BUFFER_DATA (buf), GST_BUFFER_MAXSIZE (buf));
   /* cast to unsigned long, since there's no gportable way to print
    * guint64 as hex */
-  GST_DEBUG (0, "unmapped region %08lx+%08lx at %p", 
+  GST_LOG_OBJECT (src, "unmapped region %08lx+%08lx at %p", 
 		  (unsigned long) GST_BUFFER_OFFSET (buf),
 		  (unsigned long) GST_BUFFER_MAXSIZE (buf), 
 		  GST_BUFFER_DATA (buf));
 
   GST_BUFFER_DATA (buf) = NULL;
 
+  g_object_unref (src);
   gst_buffer_default_free (buf);
 }
 
@@ -388,7 +412,7 @@ gst_filesrc_map_region (GstFileSrc *src, off_t offset, size_t size)
 
   g_return_val_if_fail (offset >= 0, NULL);
 
-  fs_print  ("mapping region %08llx+%08lx from file into memory\n",offset,(unsigned long)size);
+  GST_LOG_OBJECT (src, "mapping region %08llx+%08lx from file into memory",offset,(unsigned long)size);
   mmapregion = mmap (NULL, size, PROT_READ, MAP_SHARED, src->fd, offset);
 
   if (mmapregion == NULL) {
@@ -396,11 +420,11 @@ gst_filesrc_map_region (GstFileSrc *src, off_t offset, size_t size)
     return NULL;
   }
   else if (mmapregion == MAP_FAILED) {
-    GST_DEBUG (0, "mmap (0x%lx, %d, 0x%llx) : %s",
+    GST_WARNING_OBJECT (src, "mmap (0x%08lx, %d, 0x%llx) failed: %s",
  	     (unsigned long)size, src->fd, offset, strerror (errno));
     return NULL;
   }
-  GST_DEBUG (0, "mapped region %08lx+%08lx from file into memory at %p", 
+  GST_LOG_OBJECT (src, "mapped region %08lx+%08lx from file into memory at %p", 
 		  (unsigned long)offset, (unsigned long)size, mmapregion);
 
   /* time to allocate a new mapbuf */
@@ -420,6 +444,7 @@ gst_filesrc_map_region (GstFileSrc *src, off_t offset, size_t size)
   GST_BUFFER_OFFSET (buf) = offset;
   GST_BUFFER_TIMESTAMP (buf) = GST_CLOCK_TIME_NONE;
   GST_BUFFER_POOL_PRIVATE (buf) = src;
+  g_object_ref (src);
   GST_BUFFER_FREE_FUNC (buf) = (GstDataFreeFunction) gst_filesrc_free_parent_mmap;
 
   g_mutex_lock (src->map_regions_lock);
@@ -450,6 +475,7 @@ gst_filesrc_map_small_region (GstFileSrc *src, off_t offset, size_t size)
       return NULL;
 
     ret = gst_buffer_create_sub (map, offset - mapbase, size);
+    GST_BUFFER_OFFSET (ret) = GST_BUFFER_OFFSET (map) + offset - mapbase;
 
     gst_buffer_unref (map);
 
@@ -488,7 +514,7 @@ static GstBuffer *
 gst_filesrc_get_mmap (GstFileSrc *src)
 {
   GstBuffer *buf = NULL, *map;
-  size_t readsize;
+  size_t readsize, mapsize;
   off_t readend,mapstart,mapend;
   GstFileSrcRegion region;
   int i;
@@ -496,33 +522,37 @@ gst_filesrc_get_mmap (GstFileSrc *src)
   /* calculate end pointers so we don't have to do so repeatedly later */
   readsize = src->block_size;
   readend = src->curoffset + src->block_size;		/* note this is the byte *after* the read */
-  mapstart = GST_BUFFER_OFFSET(src->mapbuf);
-  mapend = mapstart + GST_BUFFER_SIZE(src->mapbuf);	/* note this is the byte *after* the map */
+  mapstart = GST_BUFFER_OFFSET (src->mapbuf);
+  mapsize = GST_BUFFER_SIZE (src->mapbuf);
+  mapend = mapstart + mapsize;			/* note this is the byte *after* the map */
 
   /* check to see if we're going to overflow the end of the file */
   if (readend > src->filelen) {
-    readsize = src->filelen - src->curoffset;
-    readend = src->curoffset + readsize;
+    if (!gst_filesrc_check_filesize (src) || readend > src->filelen) {
+      readsize = src->filelen - src->curoffset;
+      readend = src->curoffset + readsize;
+    }
   }
 
-  GST_DEBUG (0, "attempting to read %08lx, %08lx, %08lx, %08lx", 
-		  (unsigned long)readsize, (unsigned long)readend,
-		  (unsigned long)mapstart, (unsigned long)mapend);
+  GST_LOG ("attempting to read %08lx, %08lx, %08lx, %08lx", 
+	   (unsigned long)readsize, (unsigned long)readend,
+	   (unsigned long)mapstart, (unsigned long)mapend);
 
   /* if the start is past the mapstart */
   if (src->curoffset >= mapstart) {
     /* if the end is before the mapend, the buffer is in current mmap region... */
     /* ('cause by definition if readend is in the buffer, so's readstart) */
     if (readend <= mapend) {
-      fs_print ("read buf %llu+%d lives in current mapbuf %lld+%d, creating subbuffer of mapbuf\n",
-             src->curoffset, readsize, mapstart, GST_BUFFER_SIZE(src->mapbuf));
-      buf = gst_buffer_create_sub (src->mapbuf, src->curoffset - GST_BUFFER_OFFSET(src->mapbuf),
+      GST_LOG_OBJECT (src, "read buf %llu+%d lives in current mapbuf %lld+%d, creating subbuffer of mapbuf",
+             src->curoffset, readsize, mapstart, mapsize);
+      buf = gst_buffer_create_sub (src->mapbuf, src->curoffset - mapstart,
                                    readsize);
+      GST_BUFFER_OFFSET (buf) = src->curoffset;
 
     /* if the start actually is within the current mmap region, map an overlap buffer */
     } else if (src->curoffset < mapend) {
-      fs_print ("read buf %llu+%d starts in mapbuf %d+%d but ends outside, creating new mmap\n",
-             src->curoffset,readsize,GST_BUFFER_OFFSET(src->mapbuf),GST_BUFFER_SIZE(src->mapbuf));
+      GST_LOG_OBJECT (src, "read buf %llu+%d starts in mapbuf %d+%d but ends outside, creating new mmap",
+             (unsigned long long) src->curoffset, (gint) readsize, (gint) mapstart, (gint) mapsize);
       buf = gst_filesrc_map_small_region (src, src->curoffset, readsize);
       if (buf == NULL)
         return NULL;
@@ -536,8 +566,8 @@ gst_filesrc_get_mmap (GstFileSrc *src)
     /* either the read buffer overlaps the start of the mmap region */
     /* or the read buffer fully contains the current mmap region    */
     /* either way, it's really not relevant, we just create a new region anyway*/
-    fs_print ("read buf %llu+%d starts before mapbuf %d+%d, but overlaps it\n",
-             src->curoffset,readsize,GST_BUFFER_OFFSET(src->mapbuf),GST_BUFFER_SIZE(src->mapbuf));
+    GST_LOG_OBJECT (src, "read buf %llu+%d starts before mapbuf %d+%d, but overlaps it",
+             (unsigned long long) src->curoffset, (gint) readsize, (gint) mapstart, (gint) mapsize);
     buf = gst_filesrc_map_small_region (src, src->curoffset, readsize);
     if (buf == NULL)
       return NULL;
@@ -546,7 +576,7 @@ gst_filesrc_get_mmap (GstFileSrc *src)
   /* then deal with the case where the read buffer is totally outside */
   if (buf == NULL) {
     /* first check to see if there's a map that covers the right region already */
-    fs_print ("searching for mapbuf to cover %llu+%d\n",src->curoffset,readsize);
+    GST_LOG_OBJECT (src, "searching for mapbuf to cover %llu+%d",src->curoffset,readsize);
     region.offset = src->curoffset;
     region.size = readsize;
     map = g_tree_search (src->map_regions,
@@ -555,14 +585,16 @@ gst_filesrc_get_mmap (GstFileSrc *src)
 
     /* if we found an exact match, subbuffer it */
     if (map != NULL) {
-      fs_print ("found mapbuf at %d+%d, creating subbuffer\n",GST_BUFFER_OFFSET(map),GST_BUFFER_SIZE(map));
+      GST_LOG_OBJECT (src, "found mapbuf at %"G_GUINT64_FORMAT"+%u, creating subbuffer",
+		      GST_BUFFER_OFFSET (map), GST_BUFFER_SIZE (map));
       buf = gst_buffer_create_sub (map, src->curoffset - GST_BUFFER_OFFSET(map), readsize);
+      GST_BUFFER_OFFSET (buf) = src->curoffset;
 
     /* otherwise we need to create something out of thin air */
     } else {
       /* if the read buffer crosses a mmap region boundary, create a one-off region */
       if ((src->curoffset / src->mapsize) != (readend / src->mapsize)) {
-        fs_print ("read buf %llu+%d crosses a %d-byte boundary, creating a one-off\n",
+        GST_LOG_OBJECT (src, "read buf %llu+%d crosses a %d-byte boundary, creating a one-off",
                src->curoffset,readsize,src->mapsize);
         buf = gst_filesrc_map_small_region (src, src->curoffset, readsize);
 	if (buf == NULL)
@@ -573,7 +605,7 @@ gst_filesrc_get_mmap (GstFileSrc *src)
 	size_t mapsize;
 
         off_t nextmap = src->curoffset - (src->curoffset % src->mapsize);
-        fs_print ("read buf %llu+%d in new mapbuf at %llu+%d, mapping and subbuffering\n",
+        GST_LOG_OBJECT (src, "read buf %llu+%d in new mapbuf at %llu+%d, mapping and subbuffering",
                src->curoffset, readsize, nextmap, src->mapsize);
         /* first, we're done with the old mapbuf */
         gst_buffer_unref(src->mapbuf);
@@ -581,7 +613,7 @@ gst_filesrc_get_mmap (GstFileSrc *src)
 
 	/* double the mapsize as long as the readsize is smaller */
 	while (readsize - (src->curoffset - nextmap) > mapsize) {
-          fs_print ("readsize smaller then mapsize %08x %d\n", readsize, mapsize);
+          GST_LOG_OBJECT (src, "readsize smaller then mapsize %08x %d", readsize, mapsize);
           mapsize <<=1;
 	}
         /* create a new one */
@@ -591,6 +623,7 @@ gst_filesrc_get_mmap (GstFileSrc *src)
 
         /* subbuffer it */
         buf = gst_buffer_create_sub (src->mapbuf, src->curoffset - nextmap, readsize);
+        GST_BUFFER_OFFSET (buf) = GST_BUFFER_OFFSET (src->mapbuf) + src->curoffset - nextmap;
       }
     }
   }
@@ -604,6 +637,7 @@ gst_filesrc_get_mmap (GstFileSrc *src)
   }
 
   /* we're done, return the buffer */
+  g_assert (src->curoffset == GST_BUFFER_OFFSET (buf));
   src->curoffset += GST_BUFFER_SIZE(buf);
   return buf;
 }
@@ -617,7 +651,9 @@ gst_filesrc_get_read (GstFileSrc *src)
 
   readsize = src->block_size;
   if (src->curoffset + readsize > src->filelen) {
-    readsize = src->filelen - src->curoffset;
+    if (!gst_filesrc_check_filesize (src) || src->curoffset + readsize > src->filelen) {
+      readsize = src->filelen - src->curoffset;
+    }
   }
 
   buf = gst_buffer_new_and_alloc (readsize);
@@ -639,7 +675,7 @@ gst_filesrc_get_read (GstFileSrc *src)
   return buf;
 }
 
-static GstBuffer *
+static GstData *
 gst_filesrc_get (GstPad *pad)
 {
   GstFileSrc *src;
@@ -653,40 +689,59 @@ gst_filesrc_get (GstPad *pad)
     GstEvent *event;
 
     src->seek_happened = FALSE;
-    GST_DEBUG (GST_CAT_EVENT, "filesrc sending discont");
+    GST_DEBUG_OBJECT (src, "sending discont");
     event = gst_event_new_discontinuous (FALSE, GST_FORMAT_BYTES, src->curoffset, NULL);
     src->need_flush = FALSE;
-    return GST_BUFFER (event);
+    return GST_DATA (event);
   }
   /* check for flush */
   if (src->need_flush) {
     src->need_flush = FALSE;
-    GST_DEBUG (GST_CAT_EVENT, "filesrc sending flush");
-    return GST_BUFFER (gst_event_new_flush ());
+    GST_DEBUG_OBJECT (src, "sending flush");
+    return GST_DATA (gst_event_new_flush ());
   }
 
   /* check for EOF */
+  g_assert (src->curoffset <= src->filelen);
   if (src->curoffset == src->filelen) {
-    GST_DEBUG (0, "filesrc eos %" G_GINT64_FORMAT
-		  " %" G_GINT64_FORMAT, src->curoffset, src->filelen);
-    gst_element_set_eos (GST_ELEMENT (src));
-    return GST_BUFFER (gst_event_new (GST_EVENT_EOS));
+    if (!gst_filesrc_check_filesize (src) || src->curoffset >= src->filelen) {
+      GST_DEBUG_OBJECT (src, "eos %" G_GINT64_FORMAT" %" G_GINT64_FORMAT,
+		src->curoffset, src->filelen);
+      gst_element_set_eos (GST_ELEMENT (src));
+      return GST_DATA (gst_event_new (GST_EVENT_EOS));
+    }
   }
 
   if (src->using_mmap){
-    return gst_filesrc_get_mmap (src);
+    return GST_DATA (gst_filesrc_get_mmap (src));
   }else{
-    return gst_filesrc_get_read (src);
+    return GST_DATA (gst_filesrc_get_read (src));
   }
 }
 
+/* TRUE if the filesize of the file was updated */
+static gboolean
+gst_filesrc_check_filesize (GstFileSrc *src)
+{
+  struct stat stat_results;
+  
+  g_return_val_if_fail (GST_FLAG_IS_SET (src ,GST_FILESRC_OPEN), FALSE);
+
+  fstat(src->fd, &stat_results);
+  GST_DEBUG_OBJECT (src, "checked filesize on %s (was %"G_GUINT64_FORMAT", is %"G_GUINT64_FORMAT")", 
+	  src->filename, src->filelen, (guint64) stat_results.st_size);
+  if (src->filelen == (guint64) stat_results.st_size)
+    return FALSE;
+  src->filelen = stat_results.st_size;
+  return TRUE;
+}
 /* open the file and mmap it, necessary to go to READY state */
 static gboolean 
 gst_filesrc_open_file (GstFileSrc *src)
 {
   g_return_val_if_fail (!GST_FLAG_IS_SET (src ,GST_FILESRC_OPEN), FALSE);
 
-  GST_DEBUG(0, "opening file %s",src->filename);
+  GST_INFO_OBJECT (src, "opening file %s",src->filename);
 
   /* open the file */
   src->fd = open (src->filename, O_RDONLY);
@@ -790,6 +845,7 @@ gst_filesrc_srcpad_query (GstPad *pad, GstQueryType type,
       if (*format != GST_FORMAT_BYTES) {
 	return FALSE;
       }
+      gst_filesrc_check_filesize (src);
       *value = src->filelen;
       break;
     case GST_QUERY_POSITION:
@@ -818,12 +874,12 @@ gst_filesrc_srcpad_event (GstPad *pad, GstEvent *event)
 {
   GstFileSrc *src = GST_FILESRC (GST_PAD_PARENT (pad));
 
-  GST_DEBUG(0, "event %d", GST_EVENT_TYPE (event));
+  GST_DEBUG_OBJECT (src, "event %d", GST_EVENT_TYPE (event));
 
   switch (GST_EVENT_TYPE (event)) {
     case GST_EVENT_SEEK:
     {
-      guint64 offset;
+      gint64 offset;
 
       if (GST_EVENT_SEEK_FORMAT (event) != GST_FORMAT_BYTES) {
 	goto error;
@@ -833,22 +889,27 @@ gst_filesrc_srcpad_event (GstPad *pad, GstEvent *event)
 
       switch (GST_EVENT_SEEK_METHOD (event)) {
         case GST_SEEK_METHOD_SET:
-          if (offset > src->filelen) 
-	    goto error;
+          if (offset > src->filelen && (!gst_filesrc_check_filesize (src) || offset > src->filelen)) {
+	      goto error;
+	  }
           src->curoffset = offset;
-          GST_DEBUG(0, "seek set pending to %" G_GINT64_FORMAT, src->curoffset);
+          GST_DEBUG_OBJECT (src, "seek set pending to %" G_GINT64_FORMAT, src->curoffset);
 	  break;
         case GST_SEEK_METHOD_CUR:
           if (offset + src->curoffset > src->filelen) 
-	    goto error;
+	    if (!gst_filesrc_check_filesize (src) || offset + src->curoffset > src->filelen)
+	      goto error;
           src->curoffset += offset;
-          GST_DEBUG(0, "seek cur pending to %" G_GINT64_FORMAT, src->curoffset);
+          GST_DEBUG_OBJECT (src, "seek cur pending to %" G_GINT64_FORMAT, src->curoffset);
 	  break;
         case GST_SEEK_METHOD_END:
-          if (ABS (offset) > src->filelen) 
+          if (ABS (offset) > src->filelen) {
+	    if (!gst_filesrc_check_filesize (src) || ABS (offset) > src->filelen)
+	      goto error;
 	    goto error;
+	  }
           src->curoffset = src->filelen - ABS (offset);
-          GST_DEBUG(0, "seek end pending to %" G_GINT64_FORMAT, src->curoffset);
+          GST_DEBUG_OBJECT (src, "seek end pending to %" G_GINT64_FORMAT, src->curoffset);
 	  break;
 	default:
           goto error;
@@ -879,3 +940,55 @@ error:
   gst_event_unref (event);
   return FALSE;
 }
+
+/*** GSTURIHANDLER INTERFACE *************************************************/
+
+static guint
+gst_filesrc_uri_get_type (void)
+{
+  return GST_URI_SRC;
+}
+static gchar **
+gst_filesrc_uri_get_protocols(void)
+{
+  static gchar *protocols[] = {"file", NULL};
+  return protocols;
+}
+static const gchar *
+gst_filesrc_uri_get_uri (GstURIHandler *handler)
+{
+  GstFileSrc *src = GST_FILESRC (handler);
+  
+  return src->uri;
+}
+static gboolean
+gst_filesrc_uri_set_uri (GstURIHandler *handler, const gchar *uri)
+{
+  gchar *protocol, *location;
+  gboolean ret;
+  GstFileSrc *src = GST_FILESRC (handler);
+
+  protocol = gst_uri_get_protocol (uri);
+  if (strcmp (protocol, "file") != 0) {
+    g_free (protocol);
+    return FALSE;
+  }
+  g_free (protocol);
+  location = gst_uri_get_location (uri);
+  ret = gst_filesrc_set_location (src, location);
+  g_free (location);
+
+  return ret;
+}
+
+static void
+gst_filesrc_uri_handler_init (gpointer g_iface, gpointer iface_data)
+{
+  GstURIHandlerInterface *iface = (GstURIHandlerInterface *) g_iface;
+
+  iface->get_type = gst_filesrc_uri_get_type;
+  iface->get_protocols = gst_filesrc_uri_get_protocols;
+  iface->get_uri = gst_filesrc_uri_get_uri;
+  iface->set_uri = gst_filesrc_uri_set_uri;
+}
+
