@@ -27,42 +27,43 @@
 #include <glib.h>
 #include <popt.h>
 
-#include <gst/gstversion.h>
+#include <gst/gstenumtypes.h>
 #include <gst/gsttypes.h>
+#include <gst/gstversion.h>
 
-#include <gst/gstinfo.h>
-#include <gst/gstobject.h>
-#include <gst/gstpad.h>
+#include <gst/gstbin.h>
 #include <gst/gstbuffer.h>
 #include <gst/gstbufferpool-default.h>
+#include <gst/gstcaps.h>
+#include <gst/gstclock.h>
 #include <gst/gstcpu.h>
 #include <gst/gstelement.h>
-#include <gst/gstbin.h>
+#include <gst/gstevent.h>
 #include <gst/gstindex.h>
+#include <gst/gstinfo.h>
+#include <gst/gstinterface.h>
+#include <gst/gstobject.h>
+#include <gst/gstpad.h>
 #include <gst/gstpipeline.h>
-#include <gst/gstthread.h>
-#include <gst/gsttypefind.h>
-#include <gst/gstautoplug.h>
-#include <gst/gstcaps.h>
-#include <gst/gststructure.h>
-#include <gst/gstprops.h>
 #include <gst/gstplugin.h>
+#include <gst/gstprops.h>
+#include <gst/gstscheduler.h>
+#include <gst/gststructure.h>
+#include <gst/gstsystemclock.h>
+#include <gst/gsttag.h>
+#include <gst/gsttaginterface.h>
+#include <gst/gstthread.h>
+#include <gst/gsttrace.h>
+#include <gst/gsttypefind.h>
 #include <gst/gsturi.h>
 #include <gst/gsturitype.h>
 #include <gst/gstutils.h>
-#include <gst/gsttrace.h>
-#include <gst/gstxml.h>
-#include <gst/gstscheduler.h>
-#include <gst/gstevent.h>
-#include <gst/gstclock.h>
-#include <gst/gstsystemclock.h>
-#include <gst/gstinterface.h>
 #include <gst/gstvalue.h>
+#include <gst/gstxml.h>
 
 #include <gst/gstparse.h>
 #include <gst/gstregistry.h>
 #include <gst/gstregistrypool.h>
-#include <gst/gstenumtypes.h>
 
 /* API compatibility stuff */
 #include <gst/gstcompat.h>
@@ -78,8 +79,8 @@ void		gst_init_with_popt_table	(int *argc, char **argv[],
 gboolean	gst_init_check_with_popt_table	(int *argc, char **argv[],
 						 const struct poptOption
 						 *popt_options);
-const struct
-poptOption*	gst_init_get_popt_table		(void);
+G_CONST_RETURN struct poptOption*
+		gst_init_get_popt_table		(void);
 
 void		gst_use_threads			(gboolean use_threads);
 gboolean	gst_has_threads			(void);
