@@ -22,25 +22,32 @@
 
 #include <sys/time.h>
 
-/* #define GST_DEBUG_ENABLED */
 #include "gst_private.h"
 
 #include "gstclock.h"
-#include "gstlog.h"
+#include "gstinfo.h"
 #include "gstmemchunk.h"
 
+#ifndef GST_DISABLE_TRACE
+/* #define GST_WITH_ALLOC_TRACE */
+#include "gsttrace.h"
+static GstAllocTrace *_gst_clock_entry_trace;
+#endif
+
 #define DEFAULT_MAX_DIFF	(2 * GST_SECOND)
 
 enum {
   ARG_0,
   ARG_STATS,
-  ARG_MAX_DIFF,
+  ARG_MAX_DIFF
 };
 
-static GstMemChunk *_gst_clock_entries_chunk;
+static GstMemChunk   *_gst_clock_entries_chunk;
 
 static void		gst_clock_class_init		(GstClockClass *klass);
 static void		gst_clock_init			(GstClock *clock);
+static void 		gst_clock_dispose 		(GObject *object);
+
 static void             gst_clock_set_property		(GObject *object, guint prop_id, 
 							 const GValue *value, GParamSpec *pspec);
 static void             gst_clock_get_property		(GObject *object, guint prop_id, 
@@ -51,9 +58,6 @@ static void		gst_clock_update_stats		(GstClock *clock);
 static GstObjectClass *parent_class = NULL;
 /* static guint gst_clock_signals[LAST_SIGNAL] = { 0 }; */
 
-static GMutex *_gst_clock_mutex;
-static GCond  *_gst_clock_cond;
-
 static GstClockID
 gst_clock_entry_new (GstClock *clock, GstClockTime time, 
 		     GstClockTime interval, GstClockEntryType type)
@@ -61,6 +65,9 @@ gst_clock_entry_new (GstClock *clock, GstClockTime time,
   GstClockEntry *entry;
 
   entry = gst_mem_chunk_alloc (_gst_clock_entries_chunk);
+#ifndef GST_DISABLE_TRACE
+  gst_alloc_trace_new (_gst_clock_entry_trace, entry);
+#endif
 
   entry->clock = clock;
   entry->time = time;
@@ -109,7 +116,7 @@ gst_clock_new_periodic_id (GstClock *clock, GstClockTime start_time,
                            GstClockTime interval)
 {
   g_return_val_if_fail (GST_IS_CLOCK (clock), NULL);
-  g_return_val_if_fail (start_time != GST_CLOCK_TIME_NONE, NULL);
+  g_return_val_if_fail (GST_CLOCK_TIME_IS_VALID (start_time), NULL);
   g_return_val_if_fail (interval != 0, NULL);
 
   return gst_clock_entry_new (clock, 
@@ -159,7 +166,8 @@ gst_clock_id_wait (GstClockID id, GstClockTimeDiff *jitter)
   entry = (GstClockEntry *) id;
   requested = GST_CLOCK_ENTRY_TIME (entry);
 
-  if (requested == GST_CLOCK_TIME_NONE) {
+  if (! GST_CLOCK_TIME_IS_VALID (requested)) {
+    GST_CAT_DEBUG (GST_CAT_CLOCK, "invalid time requested, returning _TIMEOUT");
     return GST_CLOCK_TIMEOUT;
   }
 
@@ -173,10 +181,12 @@ gst_clock_id_wait (GstClockID id, GstClockTimeDiff *jitter)
     clock->entries = g_list_prepend (clock->entries, entry);
     GST_UNLOCK (clock);
 
+    GST_CAT_DEBUG (GST_CAT_CLOCK, "waiting on clock");
     do {
       res = cclass->wait (clock, entry);
     }
     while (res == GST_CLOCK_ENTRY_RESTART);
+    GST_CAT_DEBUG (GST_CAT_CLOCK, "done waiting");
 
     GST_LOCK (clock);
     clock->entries = g_list_remove (clock->entries, entry);
@@ -221,7 +231,7 @@ gst_clock_id_wait_async (GstClockID id,
   entry = (GstClockEntry *) id;
   clock = entry->clock;
 
-  if (GST_CLOCK_ENTRY_TIME (entry) == GST_CLOCK_TIME_NONE) {
+  if (! GST_CLOCK_TIME_IS_VALID (GST_CLOCK_ENTRY_TIME (entry))) {
     (func) (clock, GST_CLOCK_TIME_NONE, id, user_data);
     return GST_CLOCK_TIMEOUT;
   }
@@ -281,6 +291,9 @@ gst_clock_id_free (GstClockID id)
 {
   g_return_if_fail (id != NULL);
 
+#ifndef GST_DISABLE_TRACE
+  gst_alloc_trace_free (_gst_clock_entry_trace, id);
+#endif
   gst_mem_chunk_free (_gst_clock_entries_chunk, id);
 }
 
@@ -354,9 +367,11 @@ gst_clock_class_init (GstClockClass *klass)
                      sizeof (GstClockEntry), sizeof (GstClockEntry) * 32,
                      G_ALLOC_AND_FREE);
 
-  _gst_clock_mutex = g_mutex_new ();
-  _gst_clock_cond  = g_cond_new ();
+#ifndef GST_DISABLE_TRACE
+  _gst_clock_entry_trace = gst_alloc_trace_register (GST_CLOCK_ENTRY_TRACE_NAME);
+#endif
 
+  gobject_class->dispose      = GST_DEBUG_FUNCPTR (gst_clock_dispose);
   gobject_class->set_property = GST_DEBUG_FUNCPTR (gst_clock_set_property);
   gobject_class->get_property = GST_DEBUG_FUNCPTR (gst_clock_get_property);
 
@@ -385,6 +400,17 @@ gst_clock_init (GstClock *clock)
   clock->active_cond = g_cond_new ();
 }
 
+static void
+gst_clock_dispose (GObject *object)
+{
+  GstClock *clock = GST_CLOCK (object);
+
+  g_mutex_free (clock->active_mutex);
+  g_cond_free (clock->active_cond);
+
+  G_OBJECT_CLASS (parent_class)->dispose (object);
+}
+
 /**
  * gst_clock_set_speed
  * @clock: a #GstClock to modify
@@ -440,8 +466,8 @@ gst_clock_set_resolution (GstClock *clock, guint64 resolution)
 {
   GstClockClass *cclass;
 
-  g_return_val_if_fail (GST_IS_CLOCK (clock), 0LL);
-  g_return_val_if_fail (resolution != 0, 0LL);
+  g_return_val_if_fail (GST_IS_CLOCK (clock), G_GINT64_CONSTANT (0));
+  g_return_val_if_fail (resolution != 0, G_GINT64_CONSTANT (0));
 
   cclass = GST_CLOCK_GET_CLASS (clock);
 
@@ -464,14 +490,14 @@ gst_clock_get_resolution (GstClock *clock)
 {
   GstClockClass *cclass;
 
-  g_return_val_if_fail (GST_IS_CLOCK (clock), 0LL);
+  g_return_val_if_fail (GST_IS_CLOCK (clock), G_GINT64_CONSTANT (0));
 
   cclass = GST_CLOCK_GET_CLASS (clock);
 
   if (cclass->get_resolution)
     return cclass->get_resolution (clock);
 
-  return 1LL;
+  return G_GINT64_CONSTANT (1);
 }
 
 /**
@@ -485,7 +511,7 @@ gst_clock_get_resolution (GstClock *clock)
 void
 gst_clock_set_active (GstClock *clock, gboolean active)
 {
-  GstClockTime time = 0LL;
+  GstClockTime time = G_GINT64_CONSTANT (0);
   GstClockClass *cclass;
 
   g_return_if_fail (GST_IS_CLOCK (clock));
@@ -540,7 +566,7 @@ gst_clock_is_active (GstClock *clock)
 void
 gst_clock_reset (GstClock *clock)
 {
-  GstClockTime time = 0LL;
+  GstClockTime time = G_GINT64_CONSTANT (0);
   GstClockClass *cclass;
 
   g_return_if_fail (GST_IS_CLOCK (clock));
@@ -554,7 +580,7 @@ gst_clock_reset (GstClock *clock)
   GST_LOCK (clock);
   clock->active = FALSE;
   clock->start_time = time;
-  clock->last_time = 0LL;
+  clock->last_time = G_GINT64_CONSTANT (0);
   g_list_foreach (clock->entries, (GFunc) gst_clock_reschedule_func, NULL);
   GST_UNLOCK (clock);
 }
@@ -573,31 +599,19 @@ gst_clock_reset (GstClock *clock)
 gboolean
 gst_clock_handle_discont (GstClock *clock, guint64 time)
 {
-  GstClockTime itime = 0LL;
+  GstClockTime itime = G_GINT64_CONSTANT (0);
+  GstClockClass *cclass = GST_CLOCK_GET_CLASS (clock);;
 
-  GST_DEBUG (GST_CAT_CLOCK, "clock discont %" G_GUINT64_FORMAT
+  GST_CAT_DEBUG (GST_CAT_CLOCK, "clock discont %" G_GUINT64_FORMAT
 		            " %" G_GUINT64_FORMAT " %d",
 			    time, clock->start_time, clock->accept_discont);
 
-  if (time == GST_CLOCK_TIME_NONE)
+  if (! GST_CLOCK_TIME_IS_VALID (time))
     return TRUE;
 
   GST_LOCK (clock);
-  if (clock->accept_discont) {
-    GstClockClass *cclass;
-
-    cclass = GST_CLOCK_GET_CLASS (clock);
-	  
-    if (cclass->get_internal_time) {
-      itime = cclass->get_internal_time (clock);
-    }
-  }
-  else {
-    GST_UNLOCK (clock);
-    GST_DEBUG (GST_CAT_CLOCK, "clock discont refused %" G_GUINT64_FORMAT
-		              " %" G_GUINT64_FORMAT,
-			      time, clock->start_time);
-    return FALSE;
+  if (cclass->get_internal_time) {
+    itime = cclass->get_internal_time (clock);
   }
 
   clock->start_time = itime - time;
@@ -606,7 +620,7 @@ gst_clock_handle_discont (GstClock *clock, guint64 time)
   g_list_foreach (clock->entries, (GFunc) gst_clock_reschedule_func, NULL);
   GST_UNLOCK (clock);
 
-  GST_DEBUG (GST_CAT_CLOCK, "new time %" G_GUINT64_FORMAT,
+  GST_CAT_DEBUG (GST_CAT_CLOCK, "new time %" G_GUINT64_FORMAT,
 	     gst_clock_get_time (clock));
 
   g_mutex_lock (clock->active_mutex);
@@ -628,12 +642,12 @@ gst_clock_handle_discont (GstClock *clock, guint64 time)
 GstClockTime
 gst_clock_get_time (GstClock *clock)
 {
-  GstClockTime ret = 0LL;
+  GstClockTime ret = G_GINT64_CONSTANT (0);
 
-  g_return_val_if_fail (GST_IS_CLOCK (clock), 0LL);
+  g_return_val_if_fail (GST_IS_CLOCK (clock), G_GINT64_CONSTANT (0));
 
   if (!clock->active) {
-    /* clock is not activen return previous time */
+    /* clock is not active return previous time */
     ret = clock->last_time;
   }
   else {
