@@ -39,12 +39,13 @@ typedef guint64 	GstClockTime;
 typedef gint64 		GstClockTimeDiff;
 typedef gpointer 	GstClockID;
 
-#define GST_CLOCK_TIME_NONE  ((guint64)-1)
+#define GST_CLOCK_TIME_NONE  		((GstClockTime)-1)
+#define GST_CLOCK_TIME_IS_VALID(time)	((time) != GST_CLOCK_TIME_NONE)
 
-#define GST_SECOND  ((guint64) G_USEC_PER_SEC * 1000LL)
-#define GST_MSECOND ((guint64) GST_SECOND / 1000LL)
-#define GST_USECOND ((guint64) GST_SECOND / 1000000LL)
-#define GST_NSECOND ((guint64) GST_SECOND / 1000000000LL)
+#define GST_SECOND  ((guint64) G_USEC_PER_SEC * G_GINT64_CONSTANT (1000))
+#define GST_MSECOND ((guint64) GST_SECOND / G_GINT64_CONSTANT (1000))
+#define GST_USECOND ((guint64) GST_SECOND / G_GINT64_CONSTANT (1000000))
+#define GST_NSECOND ((guint64) GST_SECOND / G_GINT64_CONSTANT (1000000000))
 
 #define GST_CLOCK_DIFF(s, e) 		(GstClockTimeDiff)((s) - (e))
 #define GST_TIMEVAL_TO_TIME(tv)		((tv).tv_sec * GST_SECOND + (tv).tv_usec * GST_USECOND)
@@ -54,6 +55,8 @@ G_STMT_START { 						\
   (tv).tv_usec = ((t) / GST_USECOND) % GST_MSECOND;	\
 } G_STMT_END
 
+#define GST_CLOCK_ENTRY_TRACE_NAME "GstClockEntry"
+
 typedef struct _GstClockEntry 	GstClockEntry;
 typedef struct _GstClock 	GstClock;
 typedef struct _GstClockClass 	GstClockClass;
@@ -134,7 +137,7 @@ struct _GstClock {
   GCond		*active_cond;
   gboolean	 stats;
 
-  gpointer 	 dummy[4];
+  gpointer _gst_reserved[GST_PADDING];
 };
 
 struct _GstClockClass {
@@ -155,6 +158,8 @@ struct _GstClockClass {
   GstClockEntryStatus   (*wait_async)           (GstClock *clock, GstClockEntry *entry);
   void                  (*unschedule)        	(GstClock *clock, GstClockEntry *entry);
   void                  (*unlock)            	(GstClock *clock, GstClockEntry *entry);
+
+  gpointer _gst_reserved[GST_PADDING];
 };
 
 GType           	gst_clock_get_type 		(void);
