@@ -10,7 +10,6 @@
 #define GST_DISABLE_LOADSAVE_REGISTRY 1
 #define GST_DISABLE_GST_DEBUG 1
 #define GST_DISABLE_LOADSAVE 1
-#define GST_DISABLE_AUTOPLUG 1
 #define GST_DISABLE_PARSE 1
 #define GST_DISABLE_TRACE 1
 #define GST_DISABLE_ALLOC_TRACE 1
@@ -30,9 +29,6 @@
 /* DOES NOT WORK */
 /* #undef GST_DISABLE_LOADSAVE */
 
-/* DOES NOT WORK */
-/* #undef GST_DISABLE_AUTOPLUG */
-
 /* DOES NOT WORK */
 /* #undef GST_DISABLE_PARSE */
 
