@@ -29,6 +29,8 @@
 #include "gstmemchunk.h"
 #include "gstevent.h"
 #include "gstlog.h"
+#include "gsttag.h"
+
 #ifndef GST_DISABLE_TRACE
 /* #define GST_WITH_ALLOC_TRACE */
 #include "gsttrace.h"
@@ -70,6 +72,12 @@ _gst_event_copy (GstEvent *event)
   memcpy (copy, event, sizeof (GstEvent));
   
   /* FIXME copy/ref additional fields */
+  switch (GST_EVENT_TYPE (event)) {
+    case GST_EVENT_TAG:
+      copy->event_data.structure.structure = gst_structure_copy (event->event_data.structure.structure);
+    default:
+      break;
+  }
 
   return copy;
 }
@@ -83,6 +91,8 @@ _gst_event_free (GstEvent* event)
     gst_object_unref (GST_EVENT_SRC (event));
   }
   switch (GST_EVENT_TYPE (event)) {
+    case GST_EVENT_TAG:
+      gst_tag_list_free (event->event_data.structure.structure);
     default:
       break;
   }
@@ -184,7 +194,7 @@ gst_event_new_seek (GstSeekType type, gint64 offset)
 }
 
 /**
- * gst_event_new_discontinuousv:
+ * gst_event_new_discontinuous_valist:
  * @new_media: A flag indicating a new media type starts
  * @format1: The format of the discont value
  * @var_args: more discont values and formats
@@ -197,7 +207,7 @@ gst_event_new_seek (GstSeekType type, gint64 offset)
  * Returns: A new discontinuous event.
  */
 GstEvent*
-gst_event_new_discontinuousv (gboolean new_media, GstFormat format1, va_list var_args)
+gst_event_new_discontinuous_valist (gboolean new_media, GstFormat format1, va_list var_args)
 {
   GstEvent *event;
   gint count = 0;
@@ -241,7 +251,7 @@ gst_event_new_discontinuous (gboolean new_media, GstFormat format1, ...)
 
   va_start (var_args, format1);
 
-  event = gst_event_new_discontinuousv (new_media, format1, var_args);
+  event = gst_event_new_discontinuous_valist (new_media, format1, var_args);
 
   va_end (var_args);
 
