@@ -40,65 +40,36 @@ extern GType _gst_pad_type;
 extern GType _gst_real_pad_type;
 extern GType _gst_ghost_pad_type;
 
-#define GST_TYPE_PARANOID 
-
 /* 
  * Pad base class
  */
 #define GST_TYPE_PAD			(_gst_pad_type) 
-
-#define GST_PAD_CAST(obj)		((GstPad*)(obj))
-#define GST_PAD_CLASS_CAST(klass)	((GstPadClass*)(klass))
 #define GST_IS_PAD(obj)			(G_TYPE_CHECK_INSTANCE_TYPE ((obj), GST_TYPE_PAD))
 #define GST_IS_PAD_FAST(obj)		(G_OBJECT_TYPE(obj) == GST_TYPE_REAL_PAD || \
 					 G_OBJECT_TYPE(obj) == GST_TYPE_GHOST_PAD)
 #define GST_IS_PAD_CLASS(klass)		(G_TYPE_CHECK_CLASS_TYPE ((klass), GST_TYPE_PAD))
-
-#ifdef GST_TYPE_PARANOID
-# define GST_PAD(obj)			(G_TYPE_CHECK_INSTANCE_CAST ((obj), GST_TYPE_PAD, GstPad))
-# define GST_PAD_CLASS(klass)		(G_TYPE_CHECK_CLASS_CAST ((klass), GST_TYPE_PAD, GstPadClass))
-#else
-# define GST_PAD			GST_PAD_CAST
-# define GST_PAD_CLASS			GST_PAD_CLASS_CAST
-#endif
+#define GST_PAD(obj)			(G_TYPE_CHECK_INSTANCE_CAST ((obj), GST_TYPE_PAD, GstPad))
+#define GST_PAD_CLASS(klass)		(G_TYPE_CHECK_CLASS_CAST ((klass), GST_TYPE_PAD, GstPadClass))
 
 /* 
  * Real Pads
  */
 #define GST_TYPE_REAL_PAD		(_gst_real_pad_type)
-
-#define GST_REAL_PAD_CAST(obj)		((GstRealPad*)(obj))
-#define GST_REAL_PAD_CLASS_CAST(klass)	((GstRealPadClass*)(klass))
 #define GST_IS_REAL_PAD(obj)		(G_TYPE_CHECK_INSTANCE_TYPE ((obj), GST_TYPE_REAL_PAD))
 #define GST_IS_REAL_PAD_FAST(obj)	(G_OBJECT_TYPE(obj) == GST_TYPE_REAL_PAD)
 #define GST_IS_REAL_PAD_CLASS(klass)	(G_TYPE_CHECK_CLASS_TYPE ((klass), GST_TYPE_REAL_PAD))
-
-#ifdef GST_TYPE_PARANOID
-# define GST_REAL_PAD(obj)		(G_TYPE_CHECK_INSTANCE_CAST ((obj), GST_TYPE_REAL_PAD, GstRealPad))
-# define GST_REAL_PAD_CLASS(klass)	(G_TYPE_CHECK_CLASS_CAST ((klass), GST_TYPE_REAL_PAD, GstRealPadClass))
-#else
-# define GST_REAL_PAD			GST_REAL_PAD_CAST
-# define GST_REAL_PAD_CLASS		GST_REAL_PAD_CLASS_CAST
-#endif
+#define GST_REAL_PAD(obj)		(G_TYPE_CHECK_INSTANCE_CAST ((obj), GST_TYPE_REAL_PAD, GstRealPad))
+#define GST_REAL_PAD_CLASS(klass)	(G_TYPE_CHECK_CLASS_CAST ((klass), GST_TYPE_REAL_PAD, GstRealPadClass))
 
 /* 
  * Ghost Pads
  */
 #define GST_TYPE_GHOST_PAD		(_gst_ghost_pad_type)
-
-#define GST_GHOST_PAD_CAST(obj)		((GstGhostPad*)(obj))
-#define GST_GHOST_PAD_CLASS_CAST(klass)	((GstGhostPadClass*)(klass))
 #define GST_IS_GHOST_PAD(obj)		(G_TYPE_CHECK_INSTANCE_TYPE ((obj), GST_TYPE_GHOST_PAD))
 #define GST_IS_GHOST_PAD_FAST(obj)	(G_OBJECT_TYPE(obj) == GST_TYPE_GHOST_PAD)
 #define GST_IS_GHOST_PAD_CLASS(klass)	(G_TYPE_CHECK_CLASS_TYPE ((klass), GST_TYPE_GHOST_PAD))
-
-#ifdef GST_TYPE_PARANOID
-# define GST_GHOST_PAD(obj)		(G_TYPE_CHECK_INSTANCE_CAST ((obj), GST_TYPE_GHOST_PAD, GstGhostPad))
-# define GST_GHOST_PAD_CLASS(klass)	(G_TYPE_CHECK_CLASS_CAST ((klass), GST_TYPE_GHOST_PAD, GstGhostPadClass))
-#else
-# define GST_GHOST_PAD			GST_GHOST_PAD_CAST
-# define GST_GHOST_PAD_CLASS		GST_GHOST_PAD_CLASS_CAST
-#endif
+#define GST_GHOST_PAD(obj)		(G_TYPE_CHECK_INSTANCE_CAST ((obj), GST_TYPE_GHOST_PAD, GstGhostPad))
+#define GST_GHOST_PAD_CLASS(klass)	(G_TYPE_CHECK_CLASS_CAST ((klass), GST_TYPE_GHOST_PAD, GstGhostPadClass))
 
 
 /*typedef struct _GstPad GstPad; */
@@ -173,13 +144,13 @@ struct _GstPad {
 
   GstPadTemplate 	*padtemplate;	/* the template for this pad */
 
-  GST_OBJECT_PADDING
+  gpointer _gst_reserved[GST_PADDING];
 };
 
 struct _GstPadClass {
   GstObjectClass parent_class;
 
-  GST_CLASS_PADDING
+  gpointer _gst_reserved[GST_PADDING];
 };
 
 struct _GstRealPad {
@@ -221,7 +192,7 @@ struct _GstRealPad {
 
   GstProbeDispatcher 		 probedisp;
 
-  GST_OBJECT_PADDING
+  gpointer _gst_reserved[GST_PADDING];
 };
 
 struct _GstRealPadClass {
@@ -233,7 +204,7 @@ struct _GstRealPadClass {
   void 		(*linked)		(GstPad *pad, GstPad *peer);
   void 		(*unlinked)		(GstPad *pad, GstPad *peer);
 
-  GST_CLASS_PADDING
+  gpointer _gst_reserved[GST_PADDING];
 };
 
 struct _GstGhostPad {
@@ -241,13 +212,13 @@ struct _GstGhostPad {
 
   GstRealPad *realpad;
 
-  GST_OBJECT_PADDING
+  gpointer _gst_reserved[GST_PADDING];
 };
 
 struct _GstGhostPadClass {
   GstPadClass parent_class;
 
-  GST_CLASS_PADDING
+  gpointer _gst_reserved[GST_PADDING];
 };
 
 
@@ -289,7 +260,7 @@ struct _GstGhostPadClass {
 #define GST_PAD_REALIZE(pad)		(GST_IS_REAL_PAD(pad) ? ((GstRealPad *)(pad)) : GST_GPAD_REALPAD(pad))
 #define GST_PAD_DIRECTION(pad)		GST_RPAD_DIRECTION(GST_PAD_REALIZE(pad))
 #define GST_PAD_CAPS(pad)		GST_RPAD_CAPS(GST_PAD_REALIZE(pad))
-#define GST_PAD_PEER(pad)		GST_PAD_CAST(GST_RPAD_PEER(GST_PAD_REALIZE(pad)))
+#define GST_PAD_PEER(pad)		GST_PAD(GST_RPAD_PEER(GST_PAD_REALIZE(pad)))
 
 /* Some check functions (unused?) */
 #define GST_PAD_IS_LINKED(pad)		(GST_PAD_PEER(pad) != NULL)
@@ -334,7 +305,7 @@ struct _GstPadTemplate {
   GstPadPresence   presence;
   GstCaps	  *caps;
 
-  GST_OBJECT_PADDING
+  gpointer _gst_reserved[GST_PADDING];
 };
 
 struct _GstPadTemplateClass {
@@ -343,7 +314,7 @@ struct _GstPadTemplateClass {
   /* signal callbacks */
   void (*pad_created)	(GstPadTemplate *templ, GstPad *pad);
 
-  GST_CLASS_PADDING
+  gpointer _gst_reserved[GST_PADDING];
 };
 
 #ifdef G_HAVE_ISO_VARARGS
@@ -408,7 +379,7 @@ GstPad*			gst_pad_custom_new			(GType type, const gchar *name, GstPadDirection d
 GstPad*			gst_pad_custom_new_from_template	(GType type, GstPadTemplate *templ, const gchar *name);
 
 void			gst_pad_set_name			(GstPad *pad, const gchar *name);
-const gchar*		gst_pad_get_name			(GstPad *pad);
+G_CONST_RETURN gchar*	gst_pad_get_name			(GstPad *pad);
 
 GstPadDirection		gst_pad_get_direction			(GstPad *pad);
 
@@ -438,8 +409,10 @@ void			gst_pad_set_chain_function		(GstPad *pad, GstPadChainFunction chain);
 void			gst_pad_set_get_function		(GstPad *pad, GstPadGetFunction get);
 void			gst_pad_set_event_function		(GstPad *pad, GstPadEventFunction event);
 void			gst_pad_set_event_mask_function		(GstPad *pad, GstPadEventMaskFunction mask_func);
-const GstEventMask*	gst_pad_get_event_masks			(GstPad *pad);
-const GstEventMask*	gst_pad_get_event_masks_default		(GstPad *pad);
+G_CONST_RETURN GstEventMask*
+			gst_pad_get_event_masks			(GstPad *pad);
+G_CONST_RETURN GstEventMask*
+			gst_pad_get_event_masks_default		(GstPad *pad);
 
 /* pad links */
 void			gst_pad_set_link_function		(GstPad *pad, GstPadLinkFunction link);
@@ -474,15 +447,18 @@ void			gst_pad_push				(GstPad *pad, GstData *data);
 GstData*		gst_pad_pull				(GstPad *pad);
 gboolean		gst_pad_send_event			(GstPad *pad, GstEvent *event);
 gboolean		gst_pad_event_default			(GstPad *pad, GstEvent *event);
-GstPad*			gst_pad_select				(GList *padlist);
-GstPad*			gst_pad_selectv				(GstPad *pad, ...);
+GstPad*			gst_pad_selectv				(GList *padlist);
+GstPad*			gst_pad_select				(GstPad *pad, ...);
+GstPad*			gst_pad_select_valist			(GstPad *pad, va_list varargs);
 
 
 /* convert/query/format functions */
 void			gst_pad_set_formats_function		(GstPad *pad, 
 								 GstPadFormatsFunction formats);
-const GstFormat*	gst_pad_get_formats			(GstPad *pad);
-const GstFormat*	gst_pad_get_formats_default		(GstPad *pad);
+G_CONST_RETURN GstFormat*
+			gst_pad_get_formats			(GstPad *pad);
+G_CONST_RETURN GstFormat*
+			gst_pad_get_formats_default		(GstPad *pad);
 
 void			gst_pad_set_convert_function		(GstPad *pad, GstPadConvertFunction convert);
 gboolean		gst_pad_convert				(GstPad *pad, 
@@ -494,8 +470,10 @@ gboolean		gst_pad_convert_default 		(GstPad *pad,
 
 void			gst_pad_set_query_function		(GstPad *pad, GstPadQueryFunction query);
 void			gst_pad_set_query_type_function		(GstPad *pad, GstPadQueryTypeFunction type_func);
-const GstQueryType*	gst_pad_get_query_types			(GstPad *pad);
-const GstQueryType*	gst_pad_get_query_types_default		(GstPad *pad);
+G_CONST_RETURN GstQueryType*
+			gst_pad_get_query_types			(GstPad *pad);
+G_CONST_RETURN GstQueryType*
+			gst_pad_get_query_types_default		(GstPad *pad);
 gboolean		gst_pad_query				(GstPad *pad, GstQueryType type,
 								 GstFormat *format, gint64 *value);
 gboolean 		gst_pad_query_default 			(GstPad *pad, GstQueryType type,
