@@ -2,7 +2,7 @@
  * Copyright (C) 1999,2000 Erik Walthinsen <omega@cse.ogi.edu>
  *                    2000 Wim Taymans <wim.taymans@chello.be>
  *
- * gstclock.c: Clock subsystem for maintaining time sync
+ * gstsystemclock.c: Default clock, uses the system clock
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -22,9 +22,8 @@
 
 #include <sys/time.h>
 
-/* #define GST_DEBUG_ENABLED */
 #include "gst_private.h"
-#include "gstlog.h"
+#include "gstinfo.h"
 
 #include "gstsystemclock.h"
 
@@ -32,14 +31,14 @@ static GstClock *_the_system_clock = NULL;
 
 static void			gst_system_clock_class_init	(GstSystemClockClass *klass);
 static void			gst_system_clock_init		(GstSystemClock *clock);
+static void 			gst_system_clock_dispose 	(GObject *object);
 
 static GstClockTime		gst_system_clock_get_internal_time	(GstClock *clock);
 static guint64			gst_system_clock_get_resolution (GstClock *clock);
 static GstClockEntryStatus	gst_system_clock_wait		(GstClock *clock, GstClockEntry *entry);
 static void 			gst_system_clock_unlock 	(GstClock *clock, GstClockEntry *entry);
 
-static GCond 	*_gst_sysclock_cond = NULL;
-static GMutex 	*_gst_sysclock_mutex = NULL;
+static GStaticMutex _gst_sysclock_mutex = G_STATIC_MUTEX_INIT;
 
 static GstClockClass *parent_class = NULL;
 /* static guint gst_system_clock_signals[LAST_SIGNAL] = { 0 }; */
@@ -81,18 +80,39 @@ gst_system_clock_class_init (GstSystemClockClass *klass)
 
   parent_class = g_type_class_ref (GST_TYPE_CLOCK);
 
+  gobject_class->dispose	 	= gst_system_clock_dispose;
+
   gstclock_class->get_internal_time 	= gst_system_clock_get_internal_time;
   gstclock_class->get_resolution 	= gst_system_clock_get_resolution;
   gstclock_class->wait		 	= gst_system_clock_wait;
   gstclock_class->unlock		= gst_system_clock_unlock;
-
-  _gst_sysclock_cond  = g_cond_new ();
-  _gst_sysclock_mutex = g_mutex_new ();
 }
 
 static void
 gst_system_clock_init (GstSystemClock *clock)
 {
+  clock->mutex = g_mutex_new();
+  clock->cond = g_cond_new ();
+}
+
+static void
+gst_system_clock_dispose (GObject *object)
+{
+  GstClock *clock = (GstClock *) object;
+  GstSystemClock *sysclock = (GstSystemClock *) object;
+
+  /* there are subclasses of GstSystemClock running around... */
+  if (_the_system_clock == clock) {
+    g_warning ("disposing systemclock!");
+
+    /* no parent dispose here, this is bad enough already */
+  } else {
+    G_OBJECT_CLASS (parent_class)->dispose (object);
+
+    /* FIXME: Notifying before freeing? */
+    g_cond_free (sysclock->cond);
+    g_mutex_free (sysclock->mutex);
+  }
 }
 
 /**
@@ -105,15 +125,30 @@ gst_system_clock_init (GstSystemClock *clock)
 GstClock*
 gst_system_clock_obtain (void)
 {
-  if (_the_system_clock == NULL) {
-    _the_system_clock = GST_CLOCK (g_object_new (GST_TYPE_SYSTEM_CLOCK, NULL));
-    
-    gst_object_set_name (GST_OBJECT (_the_system_clock), "GstSystemClock");
+  GstClock *clock = _the_system_clock;
 
-    gst_object_ref (GST_OBJECT (_the_system_clock));
-    gst_object_sink (GST_OBJECT (_the_system_clock));
+  if (clock == NULL) {
+    g_static_mutex_lock (&_gst_sysclock_mutex);
+    if (_the_system_clock != NULL) {
+      clock = _the_system_clock;
+      g_static_mutex_unlock (&_gst_sysclock_mutex);
+      goto have_clock;
+    }
+      
+    clock = GST_CLOCK (g_object_new (GST_TYPE_SYSTEM_CLOCK, NULL));
+    
+    gst_object_set_name (GST_OBJECT (clock), "GstSystemClock");
+
+    gst_object_ref (GST_OBJECT (clock));
+    gst_object_sink (GST_OBJECT (clock));
+
+    _the_system_clock = clock;
+    g_static_mutex_unlock (&_gst_sysclock_mutex);
   }
-  return _the_system_clock;
+
+have_clock:
+  gst_object_ref (GST_OBJECT (clock));
+  return clock;
 }
 
 static GstClockTime
@@ -138,19 +173,20 @@ gst_system_clock_wait (GstClock *clock, GstClockEntry *entry)
   GstClockEntryStatus res;
   GstClockTime current, target;
   gint64 diff;
+  GstSystemClock *sysclock = GST_SYSTEM_CLOCK (clock);
 
   current = gst_clock_get_time (clock);
   diff = GST_CLOCK_ENTRY_TIME (entry) - current;
 
-  if (ABS (diff) > clock->max_diff) {
-    g_warning ("abnormal clock request diff: ABS(%" G_GINT64_FORMAT
-               ") > %" G_GINT64_FORMAT, diff, clock->max_diff);
+  if (diff + clock->max_diff < 0) {
+    g_warning ("clock is way behind: %" G_GINT64_FORMAT
+               "s (max allowed is %" G_GINT64_FORMAT "s", -diff, clock->max_diff);
     return GST_CLOCK_ENTRY_EARLY;
   }
   
   target = gst_system_clock_get_internal_time (clock) + diff;
 
-  GST_DEBUG (GST_CAT_CLOCK, "real_target %" G_GUINT64_FORMAT
+  GST_CAT_DEBUG (GST_CAT_CLOCK, "real_target %" G_GUINT64_FORMAT
 		            " target %" G_GUINT64_FORMAT
 			    " now %" G_GUINT64_FORMAT,
                             target, GST_CLOCK_ENTRY_TIME (entry), current);
@@ -159,9 +195,9 @@ gst_system_clock_wait (GstClock *clock, GstClockEntry *entry)
     GTimeVal tv;
 
     GST_TIME_TO_TIMEVAL (target, tv);
-    g_mutex_lock (_gst_sysclock_mutex);
-    g_cond_timed_wait (_gst_sysclock_cond, _gst_sysclock_mutex, &tv);
-    g_mutex_unlock (_gst_sysclock_mutex);
+    g_mutex_lock (sysclock->mutex);
+    g_cond_timed_wait (sysclock->cond, sysclock->mutex, &tv);
+    g_mutex_unlock (sysclock->mutex);
     res = entry->status;
   }
   else {
@@ -173,7 +209,9 @@ gst_system_clock_wait (GstClock *clock, GstClockEntry *entry)
 static void
 gst_system_clock_unlock (GstClock *clock, GstClockEntry *entry)
 {
-  g_mutex_lock (_gst_sysclock_mutex);
-  g_cond_broadcast (_gst_sysclock_cond);
-  g_mutex_unlock (_gst_sysclock_mutex);
+  GstSystemClock *sysclock = GST_SYSTEM_CLOCK (clock);
+
+  g_mutex_lock (sysclock->mutex);
+  g_cond_broadcast (sysclock->cond);
+  g_mutex_unlock (sysclock->mutex);
 }
