@@ -34,9 +34,8 @@
 
 #include <gst/gst_private.h>
 #include <gst/gstelement.h>
-#include <gst/gsttype.h>
+#include <gst/gsttypefind.h>
 #include <gst/gstscheduler.h>
-#include <gst/gstautoplug.h>
 #include <gst/gsturi.h>
 #include <gst/gstinfo.h>
 
@@ -398,8 +397,8 @@ gst_xml_registry_add_path_list_func (GstXMLRegistry *registry)
     g_markup_parse_context_parse (context, text, size, &error);
 
     if (error) {
-      fprintf (stderr, "ERROR: parsing registry %s: %s\n", 
-	       registry->location, error->message);
+      GST_ERROR ("parsing registry %s: %s\n", 
+	         registry->location, error->message);
       g_free (text);
       fclose (reg);
       return;
@@ -600,7 +599,7 @@ gst_xml_registry_load (GstRegistry *registry)
     g_markup_parse_context_parse (xmlregistry->context, text, size, &error);
 
     if (error) {
-      fprintf(stderr, "ERROR: parsing registry: %s\n", error->message);
+      GST_ERROR ("parsing registry: %s\n", error->message);
       g_free (text);
       CLASS (xmlregistry)->close_func (xmlregistry);
       return FALSE;
@@ -630,13 +629,17 @@ static GstRegistryReturn
 gst_xml_registry_load_plugin (GstRegistry *registry, GstPlugin *plugin)
 {
   GError *error = NULL;
+  GstPlugin *loaded_plugin;
 
   /* FIXME: add gerror support */
-  if (!gst_plugin_load_plugin (plugin, &error)) {
+  loaded_plugin = gst_plugin_load_file (plugin->filename, &error);
+  if (!plugin) {
     if (error) {
-      g_warning ("could not load plugin %s: %s", plugin->name, error->message);
+      g_warning ("could not load plugin %s: %s", plugin->desc.name, error->message);
     }
     return GST_REGISTRY_PLUGIN_LOAD_ERROR;
+  } else if (loaded_plugin != plugin) {
+    g_critical ("how to remove plugins?");
   }
 
   return GST_REGISTRY_OK;
@@ -649,18 +652,50 @@ gst_xml_registry_parse_plugin (GMarkupParseContext *context, const gchar *tag, c
   GstPlugin *plugin = registry->current_plugin;
   
   if (!strcmp (tag, "name")) {
-    plugin->name = g_strndup (text, text_len);
+    plugin->desc.name = g_strndup (text, text_len);
   }
-  else if (!strcmp (tag, "longname")) {
-    plugin->longname = g_strndup (text, text_len);
+  else if (!strcmp (tag, "description")) {
+    plugin->desc.description = g_strndup (text, text_len);
   }
   else if (!strcmp (tag, "filename")) {
     plugin->filename = g_strndup (text, text_len);
   }
+  else if (!strcmp (tag, "version")) {
+    plugin->desc.version = g_strndup (text, text_len);
+  }
+  else if (!strcmp (tag, "license")) {
+    plugin->desc.license = g_strndup (text, text_len);
+  }
+  else if (!strcmp (tag, "package")) {
+    plugin->desc.package = g_strndup (text, text_len);
+  }
+  else if (!strcmp (tag, "origin")) {
+    plugin->desc.origin = g_strndup (text, text_len);
+  }
   
   return TRUE;
 }
 
+static void
+add_to_char_array (gchar ***array, gchar *value)
+{
+  gchar **new;
+  gchar **old = *array;
+  gint i = 0;
+    
+  /* expensive, but cycles are cheap... */
+  if (old)
+    while (old[i]) i++;
+  new = g_new0 (gchar *, i + 2);
+  new[i] = value;
+  while (i > 0) {
+    i--;
+    new[i] = old[i];
+  }
+  g_free (old);
+  *array = new;
+}
+
 static gboolean
 gst_xml_registry_parse_element_factory (GMarkupParseContext *context, const gchar *tag, const gchar *text,
                                         gsize text_len, GstXMLRegistry *registry, GError **error)
@@ -668,63 +703,75 @@ gst_xml_registry_parse_element_factory (GMarkupParseContext *context, const gcha
   GstElementFactory *factory = GST_ELEMENT_FACTORY (registry->current_feature);
 
   if (!strcmp (tag, "name")) {
-    g_free (registry->current_feature->name);
-    registry->current_feature->name = g_strndup (text, text_len);
+    gchar *name = g_strndup (text, text_len);
+    gst_plugin_feature_set_name (registry->current_feature, name);
+    g_free (name);
   }
   else if (!strcmp (tag, "longname")) {
-    g_free (factory->details->longname);
-    factory->details->longname = g_strndup (text, text_len);
+    g_free (factory->details.longname);
+    factory->details.longname = g_strndup (text, text_len);
   }
   else if (!strcmp(tag, "class")) {
-    g_free (factory->details->klass);
-    factory->details->klass = g_strndup (text, text_len);
+    g_free (factory->details.klass);
+    factory->details.klass = g_strndup (text, text_len);
   }
   else if (!strcmp(tag, "description")) {
-    g_free (factory->details->description);
-    factory->details->description = g_strndup (text, text_len);
-  }
-  else if (!strcmp(tag, "license")) {
-    g_free (factory->details->license);
-    factory->details->license = g_strndup (text, text_len);
-  }
-  else if (!strcmp(tag, "version")) {
-    g_free (factory->details->version);
-    factory->details->version = g_strndup (text, text_len);
+    g_free (factory->details.description);
+    factory->details.description = g_strndup (text, text_len);
   }
   else if (!strcmp(tag, "author")) {
-    g_free (factory->details->author);
-    factory->details->author = g_strndup (text, text_len);
-  }
-  else if (!strcmp(tag, "copyright")) {
-    g_free (factory->details->copyright);
-    factory->details->copyright = g_strndup (text, text_len);
+    g_free (factory->details.author);
+    factory->details.author = g_strndup (text, text_len);
   }
   else if (!strcmp(tag, "rank")) {
     gint rank;
     gchar *ret;
-     rank = strtol (text, &ret, 0);
+    
+    rank = strtol (text, &ret, 0);
     if (ret == text + text_len) {
-     gst_element_factory_set_rank (factory, rank);
+     gst_plugin_feature_set_rank (GST_PLUGIN_FEATURE (factory), rank);
     }
+  } else if (!strcmp (tag, "uri_type")) {
+    if (strncasecmp (text, "sink", 4) == 0) {
+      factory->uri_type = GST_URI_SINK;
+    } else if (strncasecmp (text, "source", 5) == 0) {
+      factory->uri_type = GST_URI_SRC;
+    }
+  } else if (!strcmp (tag, "uri_protocol")) {
+    add_to_char_array (&factory->uri_protocols, g_strndup (text, text_len));
   }
-  
+  else if (!strcmp(tag, "interface")) {
+    gchar *tmp = g_strndup (text, text_len);
+    __gst_element_factory_add_interface (factory, tmp);
+    g_free (tmp);
+  }
+
   return TRUE;
 }
 
 static gboolean
-gst_xml_registry_parse_type_factory (GMarkupParseContext *context, const gchar *tag, const gchar *text,
-                                     gsize text_len, GstXMLRegistry *registry, GError **error)
+gst_xml_registry_parse_type_find_factory (GMarkupParseContext *context, const gchar *tag, const gchar *text,
+					  gsize text_len, GstXMLRegistry *registry, GError **error)
 {
-  GstTypeFactory *factory = GST_TYPE_FACTORY (registry->current_feature);
+  GstTypeFindFactory *factory = GST_TYPE_FIND_FACTORY (registry->current_feature);
 
   if (!strcmp (tag, "name")) {
     registry->current_feature->name = g_strndup (text, text_len);
   }
-  else if (!strcmp (tag, "mime")) {
-    factory->mime = g_strndup (text, text_len);
+  else if (!strcmp(tag, "rank")) {
+    glong rank;
+    gchar *ret;
+     rank = strtol (text, &ret, 0);
+    if (ret == text + text_len) {
+     gst_plugin_feature_set_rank (GST_PLUGIN_FEATURE (factory), rank);
+    }
   }
-  else if (!strcmp(tag, "extensions")) {
-    factory->exts = g_strndup (text, text_len);
+  /* FIXME!!
+  else if (!strcmp (tag, "caps")) {
+    factory->caps = g_strndup (text, text_len);
+  }*/
+  else if (!strcmp(tag, "extension")) {
+    add_to_char_array (&factory->extensions, g_strndup (text, text_len));
   }
 
   return TRUE;
@@ -745,21 +792,6 @@ gst_xml_registry_parse_scheduler_factory (GMarkupParseContext *context, const gc
   return TRUE;
 }
 
-static gboolean
-gst_xml_registry_parse_autoplug_factory (GMarkupParseContext *context, const gchar *tag, const gchar *text,
-                                         gsize text_len, GstXMLRegistry *registry, GError **error)
-{
-  GstAutoplugFactory *factory = GST_AUTOPLUG_FACTORY (registry->current_feature);
-
-  if (!strcmp (tag, "name")) {
-    registry->current_feature->name = g_strndup (text, text_len);
-  }
-  else if (!strcmp (tag, "longdesc")) {
-    factory->longdesc = g_strndup (text, text_len);
-  }
-  return TRUE;
-}
-
 static gboolean
 gst_xml_registry_parse_index_factory (GMarkupParseContext *context, const gchar *tag, const gchar *text,
                                       gsize text_len, GstXMLRegistry *registry, GError **error)
@@ -775,30 +807,6 @@ gst_xml_registry_parse_index_factory (GMarkupParseContext *context, const gchar
   return TRUE;
 }
 
-static gboolean
-gst_xml_registry_parse_uri_handler (GMarkupParseContext *context, const gchar *tag, const gchar *text,
-                                    gsize text_len, GstXMLRegistry *registry, GError **error)
-{
-  GstURIHandler *handler = GST_URI_HANDLER (registry->current_feature);
-
-  if (!strcmp (tag, "name")) {
-    registry->current_feature->name = g_strndup (text, text_len);
-  }
-  else if (!strcmp (tag, "uri")) {
-    handler->uri = g_strndup (text, text_len);
-  }
-  else if (!strcmp (tag, "longdesc")) {
-    handler->longdesc = g_strndup (text, text_len);
-  }
-  else if (!strcmp (tag, "element")) {
-    handler->element = g_strndup (text, text_len);
-  }
-  else if (!strcmp (tag, "property")) {
-    handler->property = g_strndup (text, text_len);
-  }
-  return TRUE;
-}
-
 static gboolean
 gst_xml_registry_parse_padtemplate (GMarkupParseContext *context, const gchar *tag, const gchar *text,
                                     gsize text_len, GstXMLRegistry *registry, GError **error)
@@ -899,28 +907,20 @@ gst_xml_registry_start_element (GMarkupParseContext *context,
 	  if (GST_IS_ELEMENT_FACTORY (feature)) {
 	    GstElementFactory *factory = GST_ELEMENT_FACTORY (feature);
 
-	    factory->details_dynamic = TRUE;
-	    factory->details = g_new0(GstElementDetails, 1);
 	    factory->padtemplates = NULL;
 	    xmlregistry->parser = gst_xml_registry_parse_element_factory;
 	    break;
 	  }
-	  else if (GST_IS_TYPE_FACTORY (feature)) {
-	    xmlregistry->parser = gst_xml_registry_parse_type_factory;
+	  else if (GST_IS_TYPE_FIND_FACTORY (feature)) {
+	    xmlregistry->parser = gst_xml_registry_parse_type_find_factory;
 	  }
 	  else if (GST_IS_SCHEDULER_FACTORY (feature)) {
 	    xmlregistry->parser = gst_xml_registry_parse_scheduler_factory;
             GST_SCHEDULER_FACTORY (feature)->type = 0;
 	  }
-	  else if (GST_IS_AUTOPLUG_FACTORY (feature)) {
-	    xmlregistry->parser = gst_xml_registry_parse_autoplug_factory;
-	  }
 	  else if (GST_IS_INDEX_FACTORY (feature)) {
 	    xmlregistry->parser = gst_xml_registry_parse_index_factory;
 	  }
-	  else if (GST_IS_URI_HANDLER (feature)) {
-	    xmlregistry->parser = gst_xml_registry_parse_uri_handler;
-	  }
 	  else {
             g_warning ("unkown feature type");
 	  }
@@ -1087,20 +1087,11 @@ gst_xml_registry_end_element (GMarkupParseContext *context,
       break;
     case GST_XML_REGISTRY_FEATURE:
       if (!strcmp (element_name, "feature")) {
-	if (GST_IS_TYPE_FACTORY (xmlregistry->current_feature)) {
-          GstTypeFactory *factory = GST_TYPE_FACTORY (xmlregistry->current_feature);
-	  gst_type_register (factory);
-	}
 	xmlregistry->state = GST_XML_REGISTRY_PLUGIN;
 	xmlregistry->parser = gst_xml_registry_parse_plugin;
 	gst_plugin_add_feature (xmlregistry->current_plugin, xmlregistry->current_feature);
 	xmlregistry->current_feature = NULL;
       }
-      else if (!strcmp (element_name, "typefind")) {
-        GstTypeFactory *factory = GST_TYPE_FACTORY (xmlregistry->current_feature);
-
-        factory->typefindfunc = gst_type_type_find_dummy;
-      }
       break;
     case GST_XML_REGISTRY_PADTEMPLATE:
       if (!strcmp (element_name, "padtemplate")) {
@@ -1115,7 +1106,7 @@ gst_xml_registry_end_element (GMarkupParseContext *context,
 	xmlregistry->name_template = NULL;
 	xmlregistry->caps = NULL;
 
-	gst_element_factory_add_pad_template (GST_ELEMENT_FACTORY (xmlregistry->current_feature),
+	__gst_element_factory_add_pad_template (GST_ELEMENT_FACTORY (xmlregistry->current_feature),
 					      template);
 	xmlregistry->state = GST_XML_REGISTRY_FEATURE;
 	xmlregistry->parser = gst_xml_registry_parse_element_factory;
@@ -1198,7 +1189,7 @@ static void
 gst_xml_registry_error (GMarkupParseContext *context, GError *error,
                         gpointer user_data)
 {
-  g_print ("error %s\n", error->message);
+  GST_ERROR ("%s\n", error->message);
 }
 
 static void
@@ -1270,12 +1261,19 @@ gst_xml_registry_paths_text (GMarkupParseContext *context, const gchar *text,
 #define PUT_ESCAPED(tag,value) 					\
 G_STMT_START{ 							\
   const gchar *toconv = value;					\
-  if (value) {							\
+  if (toconv) {							\
     gchar *v = g_markup_escape_text (toconv, strlen (toconv));	\
     CLASS (xmlregistry)->save_func (xmlregistry, "<%s>%s</%s>\n", tag, v, tag);			\
     g_free (v);							\
   }								\
 }G_STMT_END
+#define PUT_ESCAPED_INT(tag,value)				\
+G_STMT_START{ 							\
+  gchar *save = g_strdup_printf ("%ld", (glong) value);		\
+  CLASS (xmlregistry)->save_func (xmlregistry, "<%s>%s</%s>\n", tag, save, tag);		\
+  g_free (save);      						\
+}G_STMT_END
+
 
 static gboolean
 gst_xml_registry_save_props_func (GstPropsEntry *entry, 
@@ -1381,7 +1379,7 @@ gst_xml_registry_save_caps (GstXMLRegistry *xmlregistry, GstCaps *caps)
   while (caps) {
     CLASS (xmlregistry)->save_func (xmlregistry, "<capscomp>\n");
     PUT_ESCAPED ("name", caps->name);
-    PUT_ESCAPED ("type", gst_type_find_by_id (caps->id)->mime);
+    PUT_ESCAPED ("type", gst_caps_get_mime (caps));
 
     if (caps->properties) {
       CLASS (xmlregistry)->save_func (xmlregistry, "<properties>\n");
@@ -1438,66 +1436,80 @@ gst_xml_registry_save_feature (GstXMLRegistry *xmlregistry, GstPluginFeature *fe
     
   if (GST_IS_ELEMENT_FACTORY (feature)) {
     GstElementFactory *factory = GST_ELEMENT_FACTORY (feature);
-    GList *templates;
+    GList *walk;
 
-    PUT_ESCAPED ("longname", factory->details->longname);
-    PUT_ESCAPED ("class", factory->details->klass);
-    PUT_ESCAPED ("description", factory->details->description);
-    PUT_ESCAPED ("license", factory->details->license);
-    PUT_ESCAPED ("version", factory->details->version);
-    PUT_ESCAPED ("author", factory->details->author);
-    PUT_ESCAPED ("copyright", factory->details->copyright);
+    PUT_ESCAPED ("longname", factory->details.longname);
+    PUT_ESCAPED ("class", factory->details.klass);
+    PUT_ESCAPED ("description", factory->details.description);
+    PUT_ESCAPED ("author", factory->details.author);
     
-    templates = factory->padtemplates;
+    walk = factory->padtemplates;
 
-    while (templates) {
-      GstPadTemplate *template = GST_PAD_TEMPLATE (templates->data);
+    while (walk) {
+      GstPadTemplate *template = GST_PAD_TEMPLATE (walk->data);
 
       CLASS (xmlregistry)->save_func (xmlregistry, "<padtemplate>\n");
       gst_xml_registry_save_pad_template (xmlregistry, template);
       CLASS (xmlregistry)->save_func (xmlregistry, "</padtemplate>\n");
       
-      templates = g_list_next (templates);
+      walk = g_list_next (walk);
+    }
+
+    walk = factory->interfaces;
+    while (walk) {
+      PUT_ESCAPED ("interface", (gchar *) walk->data);
+      walk = g_list_next (walk);
+    }
+
+    if (GST_URI_TYPE_IS_VALID (factory->uri_type)) {
+      gchar **protocol;
+      
+      PUT_ESCAPED ("uri_type", factory->uri_type == GST_URI_SINK ? "sink" : "source");
+      g_assert (factory->uri_protocols);
+      protocol = factory->uri_protocols;
+      while (*protocol) {
+	PUT_ESCAPED ("uri_protocol", *protocol);
+	protocol++;
+      }
     }
   }
-  else if (GST_IS_TYPE_FACTORY (feature)) {
-    GstTypeFactory *factory = GST_TYPE_FACTORY (feature);
-
-    PUT_ESCAPED ("mime", factory->mime);
-    PUT_ESCAPED ("extensions", factory->exts);
-    if (factory->typefindfunc) {
-      CLASS (xmlregistry)->save_func (xmlregistry, "<typefind/>\n");
+  else if (GST_IS_TYPE_FIND_FACTORY (feature)) {
+    GstTypeFindFactory *factory = GST_TYPE_FIND_FACTORY (feature);
+    gint i = 0;
+    /* FIXME 
+    if (factory->caps) {
+      CLASS (xmlregistry)->save_func (xmlregistry, "<caps>\n");
+      gst_xml_registry_save_caps (xmlregistry, factory->caps);
+      CLASS (xmlregistry)->save_func (xmlregistry, "</caps>\n");
+    } */
+    if (factory->extensions) {
+      while (factory->extensions[i]) {
+        PUT_ESCAPED ("extension", factory->extensions[i]);
+        i++;
+      }
     }
   }
   else if (GST_IS_SCHEDULER_FACTORY (feature)) {
     PUT_ESCAPED ("longdesc", GST_SCHEDULER_FACTORY (feature)->longdesc);
   }
-  else if (GST_IS_AUTOPLUG_FACTORY (feature)) {
-    PUT_ESCAPED ("longdesc", GST_AUTOPLUG_FACTORY (feature)->longdesc);
-  }
   else if (GST_IS_INDEX_FACTORY (feature)) {
     PUT_ESCAPED ("longdesc", GST_INDEX_FACTORY (feature)->longdesc);
   }
-  else if (GST_IS_URI_HANDLER (feature)) {
-    GstURIHandler *handler = GST_URI_HANDLER (feature);
-
-    PUT_ESCAPED ("uri", handler->uri);
-    PUT_ESCAPED ("longdesc", handler->longdesc);
-    PUT_ESCAPED ("element", handler->element);
-    PUT_ESCAPED ("property", handler->property);
-  }
   return TRUE;
 }
 
-
 static gboolean
 gst_xml_registry_save_plugin (GstXMLRegistry *xmlregistry, GstPlugin *plugin)
 {
   GList *walk;
   
-  PUT_ESCAPED ("name", plugin->name);
-  PUT_ESCAPED ("longname", plugin->longname);
+  PUT_ESCAPED ("name", plugin->desc.name);
+  PUT_ESCAPED ("description", plugin->desc.description);
   PUT_ESCAPED ("filename", plugin->filename);
+  PUT_ESCAPED ("version", plugin->desc.version);
+  PUT_ESCAPED ("license", plugin->desc.license);
+  PUT_ESCAPED ("package", plugin->desc.package);
+  PUT_ESCAPED ("origin", plugin->desc.origin);
 
   walk = plugin->features;
 
@@ -1566,6 +1578,7 @@ gst_xml_registry_rebuild_recurse (GstXMLRegistry *registry,
                                   const gchar *directory)
 {
   GDir *dir;
+  gchar *temp;
   GList *ret = NULL;
 
   dir = g_dir_open (directory, 0, NULL);
@@ -1587,13 +1600,9 @@ gst_xml_registry_rebuild_recurse (GstXMLRegistry *registry,
     }
     g_dir_close (dir);
   } else {
-    if (strstr (directory, ".so")) {
-      gchar *temp;
-
-      if ((temp = strstr (directory, ".so")) &&
-          (!strcmp (temp, ".so"))) {
-        ret = g_list_prepend (ret, gst_plugin_new (directory));
-      }
+    if ((temp = strstr (directory, G_MODULE_SUFFIX)) &&
+        (!strcmp (temp, G_MODULE_SUFFIX))) {
+      ret = g_list_prepend (ret, g_strdup (directory));
     }
   }
 
@@ -1606,6 +1615,7 @@ gst_xml_registry_rebuild (GstRegistry *registry)
   GList *walk = NULL, *plugins = NULL, *prune = NULL;
   GError *error = NULL;
   guint length;
+  GstPlugin *plugin;
   GstXMLRegistry *xmlregistry = GST_XML_REGISTRY (registry);
  
   walk = registry->paths;
@@ -1631,9 +1641,10 @@ gst_xml_registry_rebuild (GstRegistry *registry)
     walk = plugins;
     while (walk) {
       g_assert (walk->data);
-      if (gst_plugin_load_plugin (GST_PLUGIN (walk->data), NULL)) {
+      plugin = gst_plugin_load_file ((gchar *) walk->data, NULL);
+      if (plugin) {
         prune = g_list_prepend (prune, walk->data);
-        gst_registry_add_plugin (registry, GST_PLUGIN (walk->data));
+        gst_registry_add_plugin (registry, plugin);
       }
 
       walk = g_list_next (walk);
@@ -1642,6 +1653,7 @@ gst_xml_registry_rebuild (GstRegistry *registry)
     walk = prune;
     while (walk) {
       plugins = g_list_remove (plugins, walk->data);
+      g_free (walk->data);
       walk = g_list_next (walk);
     }
     g_list_free (prune);
@@ -1650,16 +1662,14 @@ gst_xml_registry_rebuild (GstRegistry *registry)
   
   walk = plugins;
   while (walk) {
-    if (gst_plugin_load_plugin (GST_PLUGIN (walk->data), &error)) {
+    if ((plugin = gst_plugin_load_file ((gchar *) walk->data, &error))) {
       g_warning ("Bizarre behavior: plugin %s actually loaded", 
-	         ((GstPlugin *) walk->data)->filename);
+	         (gchar *) walk->data);
+      gst_registry_add_plugin (registry, plugin);
     } else {
       GST_CAT_INFO (GST_CAT_PLUGIN_LOADING, "Plugin %s failed to load: %s", 
-                ((GstPlugin *) walk->data)->filename, error->message);
-      g_print ("Plugin %s failed to load\n",
-                ((GstPlugin *) walk->data)->filename);
+                (gchar *) walk->data, error->message);
 
-      g_free (((GstPlugin *) walk->data)->filename);
       g_free (walk->data);
       g_error_free (error);
       error = NULL;
